@@ -3,14 +3,16 @@
 // Purpose:     The class to store a DVD Structure (Titles/Menus)
 // Author:      Alex Thuering
 // Created:	29.01.2003
-// RCS-ID:      $Id: DVD.cpp,v 1.51 2009/03/04 20:09:57 ntalex Exp $
+// RCS-ID:      $Id: DVD.cpp,v 1.60 2009/06/15 17:59:33 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
 
 #include "DVD.h"
+#include "Cache.h"
 #include "Config.h"
 #include <wx/file.h>
+#include <wx/filename.h>
 #include <wxSVGXML/svgxmlhelpr.h>
 #include <wxVillaLib/utils.h>
 #include <wxSVG/mediadec_ffmpeg.h>
@@ -184,6 +186,22 @@ wxString Stream::GetSourceFormat() {
 
 ////////////////////////////////// Vob //////////////////////////////////////
 
+Vob::Vob(const Vob& vob) {
+	m_filename = vob.m_filename;
+	m_audioFilenames = vob.m_audioFilenames;
+	for (unsigned int i = 0; i<vob.m_subtitles.size(); i++)
+		m_subtitles.Add(new TextSub(*vob.m_subtitles[i]));
+	m_tmpFilename = vob.m_tmpFilename;
+	m_chapters = vob.m_chapters;
+	m_pause = vob.m_pause;
+	m_menu = vob.m_menu;
+	m_slideshow = vob.m_slideshow;
+	m_duration = vob.m_duration;
+	for (unsigned int i = 0; i<vob.m_streams.size(); i++)
+		m_streams.Add(new Stream(*vob.m_streams[i]));
+	m_doNotTranscode = vob.m_doNotTranscode;
+}
+
 Vob::~Vob() {
 	if (m_menu)
 		delete m_menu;
@@ -197,7 +215,7 @@ bool Vob::SetFilename(wxString value) {
 	m_filename = value;
 	wxFfmpegMediaDecoder ffmpeg;
 	bool ok = m_filename.length() && ffmpeg.Load(m_filename);
-	m_duration = ok ? ffmpeg.GetDuration() : 0;
+	m_duration = ok && ffmpeg.GetDuration() > 0 ? ffmpeg.GetDuration() : 0;
 	m_streams.RemoveAt(0, m_streams.GetCount() - m_audioFilenames.GetCount());
 	for (unsigned int stIdx = 0; stIdx < ffmpeg.GetStreamCount(); stIdx++) {
 		Stream* stream = new Stream(ffmpeg.GetStreamType(stIdx), ffmpeg.GetCodecName(stIdx));
@@ -243,8 +261,8 @@ bool Vob::AddAudioFile(wxString filename) {
 }
 
 void Vob::RemoveAudioFile(int index) {
-	m_audioFilenames.RemoveAt(index);
 	m_streams.RemoveAt(m_streams.GetCount() - m_audioFilenames.GetCount() + index);
+	m_audioFilenames.RemoveAt(index);
 }
 
 long Vob::GetChapterTime(int chapter) {
@@ -285,8 +303,17 @@ wxString Vob::GetVideoFrameUri(int chapter, long position) {
 wxSvgXmlNode* Vob::GetXML(DVDFileType type, DVD* dvd) {
 	wxSvgXmlNode* node = new wxSvgXmlNode(wxSVGXML_ELEMENT_NODE, wxT("vob"));
 	wxString fname = GetFilename();
-	if (type == DVDAUTHOR_XML && GetTmpFilename().length())
-		fname = GetTmpFilename();
+	if (type == DVDAUTHOR_XML) {
+		if (GetTmpFilename().length())
+			fname = GetTmpFilename();
+	} else {
+		wxFileName filename(fname);
+		if (filename.GetPath() == dvd->GetPath(false))
+			fname = filename.GetFullName();
+		else if (filename.GetPath().StartsWith(dvd->GetPath(false)))
+			fname = filename.GetPath().substr(dvd->GetPath(false).length() + 1) + wxFILE_SEP_PATH
+					+  filename.GetFullName();
+	}
 	if (fname.length())
 		node->AddProperty(wxT("file"), fname);
 	if (type == DVDSTYLER_XML) {
@@ -341,7 +368,7 @@ wxSvgXmlNode* Vob::GetXML(DVDFileType type, DVD* dvd) {
 		node->AddProperty(wxT("chapters"), GetChapters());
 
 	if (GetMenu() && type == DVDSTYLER_XML)
-		node->AddChild(GetMenu()->GetXML(type));
+		node->AddChild(GetMenu()->GetXML(type, dvd));
 
 	if (GetSlideshow()) {
 		if (type == DVDSTYLER_XML)
@@ -365,20 +392,32 @@ wxSvgXmlNode* Vob::GetXML(DVDFileType type, DVD* dvd) {
 	return node;
 }
 
-bool Vob::PutXML(wxSvgXmlNode* node) {
+bool Vob::PutXML(wxSvgXmlNode* node, DVD* dvd) {
 	wxString val;
 	long lval;
 
 	node->GetPropVal(wxT("file"), &val);
-	if (val.length() > 0)
+	if (val.length() > 0) {
+		wxFileName fname(val);
+		if (fname.IsRelative())
+			val = dvd->GetPath() + fname.GetFullPath();
+		else if (!wxFileExists(val) && wxFileExists(dvd->GetPath() + fname.GetFullName()))
+			val = dvd->GetPath() + fname.GetFullName();
 		SetFilename(val);
+	}
 	int stIdx = 0;
 	wxSvgXmlNode* child = node->GetChildren();
 	while (child) {
 		if (child->GetName() == wxT("video")) {
 			if (child->GetChildren() != NULL
 					&& child->GetChildren()->GetContent().length() > 0) {
-				SetFilename(child->GetChildren()->GetContent());
+				val = child->GetChildren()->GetContent();
+				wxFileName fname(val);
+				if (fname.IsRelative())
+					val = dvd->GetPath() + fname.GetFullPath();
+				else if (!wxFileExists(val) && wxFileExists(dvd->GetPath() + fname.GetFullName()))
+					val = dvd->GetPath() + fname.GetFullName();
+				SetFilename(val);
 			}
 			if (child->GetPropVal(wxT("format"), &val) && val.length() > 0
 					&& val.ToLong(&lval) && (int)m_streams.GetCount() > stIdx)
@@ -386,7 +425,13 @@ bool Vob::PutXML(wxSvgXmlNode* node) {
 		} else if (child->GetName() == wxT("audio")) {
 			if (child->GetChildren() != NULL
 					&& child->GetChildren()->GetContent().length() > 0) {
-				AddAudioFile(child->GetChildren()->GetContent());
+				val = child->GetChildren()->GetContent();
+				wxFileName fname(val);
+				if (fname.IsRelative())
+					val = dvd->GetPath() + fname.GetFullPath();
+				else if (!wxFileExists(val) && wxFileExists(dvd->GetPath() + fname.GetFullName()))
+					val = dvd->GetPath() + fname.GetFullName();
+				AddAudioFile(val);
 				stIdx = m_streams.GetCount() - 1;
 			}
 			if (child->GetPropVal(wxT("format"), &val) && val.length() > 0
@@ -398,7 +443,7 @@ bool Vob::PutXML(wxSvgXmlNode* node) {
 			GetSubtitles().Add(textsub);
 		} else if (child->GetName() == wxT("menu")) {
 			m_menu = new Menu();
-			if (!m_menu->PutXML(child))
+			if (!m_menu->PutXML(child, dvd))
 				return false;
 		} else if (child->GetName() == wxT("slideshow"))
 			m_slideshow = new Slideshow(child);
@@ -487,6 +532,15 @@ int Vob::GetSize(DVD* dvd, bool generated) {
 	return size;
 }
 
+int Vob::GetRequiredSize(DVD* dvd, Cache* cache) {
+	int size = GetSize(dvd);
+	if (GetMenu() || GetSlideshow() || GetSubtitles().Count() > 0
+			 || ((!GetDoNotTranscode() || GetAudioFilenames().Count() > 0)
+					 && cache->Find(this, dvd, false).length() == 0))
+		size = size * 2;
+	return size;
+}
+
 Vob* Vob::CreateEmptyVob(VideoFormat videoFormat, AudioFormat audioFormat) {
 	wxString filename= wxT("empty_pal_mp2.mpg");
 	if (videoFormat == vfNTSC) {
@@ -522,7 +576,7 @@ wxSvgXmlNode* Pgc::GetXML(DVDFileType type, DVD* dvd, wxString nextTitle) {
 		node->AddProperty(wxT("palette"), m_palette);
 
 	if (GetMenu() && type == DVDAUTHOR_XML)
-		GetMenu()->GetXML(type, dvd->GetPlayAllRegister(), node);
+		GetMenu()->GetXML(type, dvd, node);
 
 	if (GetSlideshow() && type == DVDAUTHOR_XML)
 		GetSlideshow()->GetXML(type, node);
@@ -544,7 +598,7 @@ wxSvgXmlNode* Pgc::GetXML(DVDFileType type, DVD* dvd, wxString nextTitle) {
 	return node;
 }
 
-bool Pgc::PutXML(wxSvgXmlNode* node) {
+bool Pgc::PutXML(wxSvgXmlNode* node, DVD* dvd) {
 	WX_CLEAR_ARRAY(GetVobs());
 
 	wxString val;
@@ -558,7 +612,7 @@ bool Pgc::PutXML(wxSvgXmlNode* node) {
 		if (child->GetName() == wxT("spumux")) {
 			// get spunode (old)
 			menu = new Menu();
-			menu->PutXML(child);
+			menu->PutXML(child, dvd);
 		} else if (child->GetName() == wxT("vob")) {
 			// get vob
 			Vob* vob;
@@ -567,7 +621,7 @@ bool Pgc::PutXML(wxSvgXmlNode* node) {
 			else
 				vob = new Vob(wxT(""));
 			menu = NULL;
-			if (vob->PutXML(child))
+			if (vob->PutXML(child, dvd))
 				GetVobs().Add(vob);
 			else {
 				delete vob;
@@ -732,7 +786,7 @@ wxSvgXmlNode* Menus::GetXML(DVDFileType type, DVD* dvd) {
 	return node;
 }
 
-bool Menus::PutXML(wxSvgXmlNode* node) {
+bool Menus::PutXML(wxSvgXmlNode* node, DVD* dvd) {
 	if (node == NULL || node->GetName() != wxT("menus"))
 		return false;
 	WX_CLEAR_ARRAY(*this);
@@ -744,7 +798,7 @@ bool Menus::PutXML(wxSvgXmlNode* node) {
 			GetVideo().PutXML(child);
 		else if (child->GetName() == wxT("pgc")) {
 			Pgc* pgc = new Pgc;
-			if (pgc->PutXML(child))
+			if (pgc->PutXML(child, dvd))
 				Add(pgc);
 			else {
 				delete pgc;
@@ -776,7 +830,7 @@ wxSvgXmlNode* Titles::GetXML(DVDFileType type, DVD* dvd, int nextTitleset) {
 	return node;
 }
 
-bool Titles::PutXML(wxSvgXmlNode* node) {
+bool Titles::PutXML(wxSvgXmlNode* node, DVD* dvd) {
 	if (node == NULL || node->GetName() != wxT("titles"))
 		return false;
 	wxSvgXmlNode* child = node->GetChildren();
@@ -787,7 +841,7 @@ bool Titles::PutXML(wxSvgXmlNode* node) {
 			GetVideo().PutXML(child);
 		else if (child->GetName() == wxT("pgc")) {
 			Pgc* pgc = new Pgc;
-			if (pgc->PutXML(child))
+			if (pgc->PutXML(child, dvd))
 				Add(pgc);
 			else {
 				delete pgc;
@@ -819,18 +873,18 @@ wxSvgXmlNode* Titleset::GetXML(DVDFileType type, DVD* dvd, int nextTitleset) {
 	return node;
 }
 
-bool Titleset::PutXML(wxSvgXmlNode* node) {
+bool Titleset::PutXML(wxSvgXmlNode* node, DVD* dvd) {
 	if (node == NULL || node->GetName() != wxT("titleset"))
 		return false;
 	wxSvgXmlNode* child = node->GetChildren();
 	while (child) {
 		if (child->GetName() == wxT("menus")) {
-			if (!GetMenus().PutXML(child)) {
+			if (!GetMenus().PutXML(child, dvd)) {
 				wxLogError(_("Can't load menus"));
 				return false;
 			}
 		} else if (child->GetName() == wxT("titles")) {
-			if (!GetTitles().PutXML(child)) {
+			if (!GetTitles().PutXML(child, dvd)) {
 				wxLogError(_("Can't load titles"));
 				return false;
 			}
@@ -864,11 +918,11 @@ wxSvgXmlNode* Vmgm::GetXML(DVDFileType type, DVD* dvd) {
 	return node;
 }
 
-bool Vmgm::PutXML(wxSvgXmlNode* node) {
+bool Vmgm::PutXML(wxSvgXmlNode* node, DVD* dvd) {
 	wxSvgXmlNode* child = node->GetChildren();
 	while (child) {
 		if (child->GetName() == wxT("menus"))
-			Menus::PutXML(child);
+			Menus::PutXML(child, dvd);
 		else if (child->GetName() == wxT("fpc"))
 			SetFpc(child->GetChildren()->GetContent());
 		child = child->GetNext();
@@ -901,14 +955,14 @@ long DVD::GetCapacityValue() {
 	if (m_capacity == dcDVD47)
 		return 4590208;
 	else if (m_capacity == dcDVD85)
-		return 83434240;
+		return 8343424;
 	return 0;
 }
 
 /** Calculates video bitrate if it set to auto */
 wxArrayInt DVD::CalculateVideoBitrate() {
 	long size = 0;
-	long fixSize = 0;
+	long fixSize = s_config.GetDvdReservedSpace();
 	long fixDuration = 0;
 	long duration = 0;
 	if (m_videoBitrateAuto)
@@ -1047,7 +1101,8 @@ bool DVD::Open(wxString fname) {
 		wxLogError(_("This is not a DVDStyler project file"));
 		return false;
 	}
-
+	
+	m_path = wxFileName(fname).GetPath();
 	root->GetPropVal(wxT("name"), &m_label);
 
 	wxString val;
@@ -1076,13 +1131,13 @@ bool DVD::Open(wxString fname) {
 	wxSvgXmlNode* child = root->GetChildren();
 	while (child) {
 		if (child->GetName() == wxT("vmgm")) {
-			if (!GetVmgm().PutXML(child)) {
+			if (!GetVmgm().PutXML(child, this)) {
 				wxLogError(_("Can't load vmgm menus"));
 				return false;
 			}
 		} else if (child->GetName() == wxT("titleset")) {
 			Titleset* titleset = new Titleset;
-			if (titleset->PutXML(child))
+			if (titleset->PutXML(child, this))
 				GetTitlesets().Add(titleset);
 			else {
 				delete titleset;
@@ -1096,6 +1151,7 @@ bool DVD::Open(wxString fname) {
 }
 
 bool DVD::Save(wxString fname) {
+	m_path = wxFileName(fname).GetPath();
 	m_playAllRegister = -1;
 	wxSvgXmlDocument xml;
 	wxSvgXmlNode* root = new wxSvgXmlNode(wxSVGXML_ELEMENT_NODE, wxT("dvdstyler"));
@@ -1137,7 +1193,7 @@ bool DVD::SaveDVDAuthor(wxString fname) {
 }
 
 long DVD::GetSize(bool generated) {
-	int size = 0;
+	int size = s_config.GetDvdReservedSpace();
 	for (int tsi = -1; tsi<(int)GetTitlesets().Count(); tsi++) {
 		for (int m=0; m<=1; m++) {
 			bool menu = m == 0;
@@ -1154,6 +1210,24 @@ long DVD::GetSize(bool generated) {
 	return size;
 }
 
+long DVD::GetRequiredSize(Cache* cache) {
+	int size = 0;
+	for (int tsi = -1; tsi<(int)GetTitlesets().Count(); tsi++) {
+		for (int m=0; m<=1; m++) {
+			bool menu = m == 0;
+			if (tsi == -1 && !menu) // "titleset -1" contains only vmMenus
+				break;
+			PgcArray& pgcs = GetPgcArray(tsi, menu);
+			for (int pgci = 0; pgci<(int)pgcs.Count(); pgci++) {
+				Pgc* pgc = pgcs[pgci];
+				for (int vobi = 0; vobi<(int)pgc->GetVobs().Count(); vobi++)
+					size += pgc->GetVobs()[vobi]->GetRequiredSize(this, cache);
+			}
+		}
+	}
+	return size;
+}
+
 bool DVD::HasPlayAllButton() {
 	for (int tsi = -1; tsi<(int)GetTitlesets().Count(); tsi++) {
 		PgcArray& pgcs = GetPgcArray(tsi, true);
