@@ -3,7 +3,7 @@
 // Purpose:     The class to store a DVD Structure (Titles/Menus)
 // Author:      Alex Thuering
 // Created:	29.01.2003
-// RCS-ID:      $Id: DVD.h,v 1.34 2008/12/30 20:48:59 ntalex Exp $
+// RCS-ID:      $Id: DVD.h,v 1.37 2009/06/13 14:45:28 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -20,6 +20,7 @@
 
 class wxSvgXmlNode;
 class DVD;
+class Cache;
 
 class TextSub
 {
@@ -151,6 +152,7 @@ public:
 	Vob(wxString filename): m_pause(0), m_menu(NULL), m_slideshow(NULL), m_doNotTranscode(false) { SetFilename(filename); }
 	Vob(Menu* menu): m_pause(0), m_menu(menu), m_slideshow(NULL), m_doNotTranscode(false) {}
 	Vob(Slideshow* slideshow): m_pause(0), m_menu(NULL), m_slideshow(slideshow), m_doNotTranscode(false) {}
+	Vob(const Vob& vob);
 	~Vob();
 	
 	/** sets name of mpeg file for this vob*/
@@ -207,13 +209,21 @@ public:
    	inline void SetDoNotTranscode(bool doNotTranscode) { m_doNotTranscode = doNotTranscode; }
 	
 	wxSvgXmlNode* GetXML(DVDFileType type, DVD* dvd);
-	bool PutXML(wxSvgXmlNode* node);
+	bool PutXML(wxSvgXmlNode* node, DVD* dvd);
 	
-	/** returns size im Kb
-      * @param generated true if vob file already generated (before runnig of dvdauthor)
-      */
+	/**
+	 * Returns size in KB
+     * @param generated true if vob file already generated (before runnig of dvdauthor)
+     * @return Size of VOB file in KB
+     */
 	int GetSize(DVD* dvd, bool generated = false);
 	
+	/**
+	 * Returns size in KB required for generation
+	 * @return Size in KB
+	 */
+	int GetRequiredSize(DVD* dvd, Cache* cache);
+	
 	static Vob* CreateEmptyVob(VideoFormat videoFormat, AudioFormat audioFormat);
 	
 private:
@@ -256,7 +266,7 @@ class Pgc
     Slideshow* GetSlideshow();
 	
 	wxSvgXmlNode*	GetXML(DVDFileType type, DVD* dvd, wxString nextTitle);
-	bool		PutXML(wxSvgXmlNode* node);
+	bool PutXML(wxSvgXmlNode* node, DVD* dvd);
 	
   protected:
 	VobArray m_vobs;
@@ -348,13 +358,13 @@ class PgcArray: public PgcArrayBase {
 class Menus: public PgcArray {
   public:
 	wxSvgXmlNode*	GetXML(DVDFileType type, DVD* dvd);
-	bool		PutXML(wxSvgXmlNode* node);
+	bool PutXML(wxSvgXmlNode* node, DVD* dvd);
 };
 
 class Titles: public PgcArray {
   public:
 	wxSvgXmlNode* GetXML(DVDFileType type, DVD* dvd, int nextTitleset);
-	bool	   PutXML(wxSvgXmlNode* node);
+	bool PutXML(wxSvgXmlNode* node, DVD* dvd);
 };
 
 class Titleset {
@@ -367,7 +377,7 @@ class Titleset {
 	inline bool IsEmpty() { return GetMenus().Count() == 0 && GetTitles().Count() == 0; }
 	
 	wxSvgXmlNode* GetXML(DVDFileType type, DVD* dvd, int nextTitleset = -1);
-	bool PutXML(wxSvgXmlNode* node);
+	bool PutXML(wxSvgXmlNode* node, DVD* dvd);
 	
   protected:
 	Menus m_menus;
@@ -382,7 +392,7 @@ class Vmgm: public Menus {
     void SetFpc(wxString fpc) { m_fpc = fpc; }
 	
     wxSvgXmlNode* GetXML(DVDFileType type, DVD* dvd);
-	bool PutXML(wxSvgXmlNode* node);
+	bool PutXML(wxSvgXmlNode* node, DVD* dvd);
 	
   protected:
 	wxString m_fpc;
@@ -405,6 +415,9 @@ class DVD {
     wxString GetLabel() { return m_label; }
     /** Sets disc label */
     void SetLabel(wxString label) { m_label = label; }
+    
+    /** Returns path of loaded project file */
+    wxString GetPath(bool withSep = true) { return withSep ? m_path + wxFILE_SEP_PATH : m_path; }
 	
     /** Returns disc capacity */
     DiscCapacity GetCapacity() { return m_capacity; }
@@ -484,12 +497,14 @@ class DVD {
     /** saves a configuration for dvdauthor */
 	bool SaveDVDAuthor(wxString fname);
 	
-	/** returns estimated size of DVD im Kb */
+	/** Returns estimated size of DVD in KB */
 	long GetSize(bool generated = false);
+	/** Returns size required for generation of DVD in KB */
+	long GetRequiredSize(Cache* cache);
 	
 	/**
-	 *  returns number of g-register that will be used to implement "play all"
-	 *  or -1 if we don't need "play all"
+	 *  Returns number of g-register that will be used to implement "play all"
+	 *  or -1 if "play all" is not used
 	 */
 	inline int GetPlayAllRegister() { return m_playAllRegister; }
 	
@@ -518,7 +533,8 @@ class DVD {
 	static wxString GetVideoFormatName(VideoFormat format);
 	static wxString GetAudioFormatName(AudioFormat format);
 	
-  protected:
+private:
+	wxString m_path;
     wxString m_label;
     DiscCapacity m_capacity;
     bool m_videoBitrateAuto;
