@@ -3,7 +3,7 @@
 // Purpose:     Main window
 // Author:      Alex Thuering
 // Created:     10.10.2003
-// RCS-ID:      $Id: MainWin.cpp,v 1.109 2009/02/15 22:27:47 ntalex Exp $
+// RCS-ID:      $Id: MainWin.cpp,v 1.116 2009/08/03 21:59:05 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -44,6 +44,7 @@
 #endif
 #include "rc/run.png.h"
 #include "rc/files.png.h"
+#include "rc/add.png.h"
 
 #define BACKGROUNDS_DIR wxFindDataDirectory(_T("backgrounds"))
 #define BUTTONS_DIR wxFindDataDirectory(_T("buttons"))
@@ -71,8 +72,23 @@ enum {
 
 BEGIN_EVENT_TABLE(MainWin, wxFrame)
     // begin wxGlade: MainWin::event_table
+    EVT_MENU(wxID_NEW, MainWin::OnNew)
+    EVT_MENU(wxID_OPEN, MainWin::OnOpen)
+    EVT_MENU(wxID_SAVE, MainWin::OnSave)
+    EVT_MENU(wxID_SAVEAS, MainWin::OnSaveAs)
+    EVT_MENU(MENU_BURN_ID, MainWin::OnBurn)
+    EVT_MENU(wxID_EXIT, MainWin::OnExit)
+    EVT_MENU(MENU_DVD_ADD_FILE_ID, MainWin::OnAddFile)
+    EVT_MENU(MENU_DVD_ADD_MENU_ID, MainWin::OnAddMenu)
+    EVT_MENU(MENU_DVD_ADD_VMMENU_ID, MainWin::OnAddVmMenu)
+    EVT_MENU(MENU_DVD_ADD_TITLESET_ID, MainWin::OnAddTitleset)
+    EVT_MENU(MENU_DVD_OPTIONS_ID, MainWin::OnDVDOptions)
+    EVT_MENU(MENU_SETTINGS_ID, MainWin::OnSettings)
     EVT_MENU(wxID_HELP_CONTENTS, MainWin::OnHelpContents)
+    EVT_MENU(wxID_ABOUT, MainWin::OnAbout)
     // end wxGlade
+    EVT_UPDATE_UI(MENU_DVD_ADD_TITLESET_ID, MainWin::OnAddTitlesetUpdateUI)
+    
 	EVT_TREE_SEL_CHANGED(-1, MainWin::OnDirSelected)
 	
 	EVT_THUMBNAILS_DCLICK(THUMBNAILS_ID, MainWin::OnThumbDoubleClick)
@@ -88,20 +104,6 @@ BEGIN_EVENT_TABLE(MainWin, wxFrame)
 	EVT_TOOL(BACKGROUNS_BT_ID, MainWin::OnBackgroundsBt)
 	EVT_TOOL(BUTTONS_BT_ID, MainWin::OnButtonsBt)
 	
-	EVT_TOOL(wxID_NEW, MainWin::OnNew)
-	EVT_TOOL(wxID_OPEN, MainWin::OnOpen)
-	EVT_TOOL(wxID_SAVE, MainWin::OnSave)
-	EVT_TOOL(wxID_SAVEAS, MainWin::OnSaveAs)
-	EVT_TOOL(MENU_BURN_ID, MainWin::OnBurn)
-	EVT_TOOL(wxID_EXIT, MainWin::OnExit)
-	EVT_MENU(MENU_DVD_ADD_MENU_ID, MainWin::OnAddMenu)
-	EVT_MENU(MENU_DVD_ADD_VMMENU_ID, MainWin::OnAddVmMenu)
-	EVT_MENU(MENU_DVD_ADD_TITLESET_ID, MainWin::OnAddTitleset)
-	EVT_UPDATE_UI(MENU_DVD_ADD_TITLESET_ID, MainWin::OnAddTitlesetUpdateUI)
-	EVT_MENU(MENU_DVD_OPTIONS_ID, MainWin::OnDVDOptions)
-	EVT_MENU(MENU_SETTINGS_ID, MainWin::OnSettings)
-	EVT_MENU(wxID_ABOUT, MainWin::OnAbout)
-	
 	EVT_MENU(THUMBMENU_BACKGROUND_ID, MainWin::OnSetBackground)
 	EVT_UPDATE_UI(THUMBMENU_BACKGROUND_ID, MainWin::OnUpdateUISetBackground)
 	EVT_MENU(REFRESH_ID, MainWin::OnRefresh)
@@ -124,13 +126,13 @@ MainWin::MainWin(): wxFrame(NULL, -1, _T(""), wxDefaultPosition, wxDefaultSize,
     wxglade_tmp_menu_1->Append(wxID_OPEN, _("&Open...\tCtrl-O"), wxEmptyString, wxITEM_NORMAL);
     wxglade_tmp_menu_1->Append(wxID_SAVE, _("&Save\tCtrl-S"), wxEmptyString, wxITEM_NORMAL);
     wxglade_tmp_menu_1->Append(wxID_SAVEAS, _("S&ave as...\tShift-Ctrl-S"), wxEmptyString, wxITEM_NORMAL);
-    wxglade_tmp_menu_1->Append(wxID_SAVEAS, _("S&ave as...\tShift-Ctrl-S"), wxEmptyString, wxITEM_NORMAL);
     wxglade_tmp_menu_1->Append(MENU_BURN_ID, _("&Burn DVD...\tF9"), wxEmptyString, wxITEM_NORMAL);
     wxglade_tmp_menu_1->AppendSeparator();
     wxglade_tmp_menu_1->Append(wxID_EXIT, _("&Exit\tAlt-X"), wxEmptyString, wxITEM_NORMAL);
     m_menubar->Append(wxglade_tmp_menu_1, _("&File"));
     wxMenu* wxglade_tmp_menu_2 = new wxMenu();
     wxMenu* wxglade_tmp_menu_2_sub = new wxMenu();
+    wxglade_tmp_menu_2_sub->Append(MENU_DVD_ADD_FILE_ID, _("&File..."), wxEmptyString, wxITEM_NORMAL);
     wxglade_tmp_menu_2_sub->Append(MENU_DVD_ADD_MENU_ID, _("&Menu"), wxEmptyString, wxITEM_NORMAL);
     wxglade_tmp_menu_2_sub->Append(MENU_DVD_ADD_VMMENU_ID, _("&vmMenu"), wxEmptyString, wxITEM_NORMAL);
     wxglade_tmp_menu_2_sub->Append(MENU_DVD_ADD_TITLESET_ID, _("&Titleset"), wxEmptyString, wxITEM_NORMAL);
@@ -167,17 +169,7 @@ MainWin::MainWin(): wxFrame(NULL, -1, _T(""), wxDefaultPosition, wxDefaultSize,
     m_menuEditor->SetTitlesetManager(m_titlesetManager);
     
     // set fiter
-    m_thumbnails->AddCustomExtention(wxT("sub"));
-    m_thumbnails->AddCustomExtention(wxT("srt"));
-    m_thumbnails->AddCustomExtention(wxT("ssa"));
-    m_thumbnails->AddCustomExtention(wxT("smi"));
-    m_thumbnails->AddCustomExtention(wxT("rt"));
-    m_thumbnails->AddCustomExtention(wxT("txt"));
-    m_thumbnails->AddCustomExtention(wxT("aqt"));
-    m_thumbnails->AddCustomExtention(wxT("jss"));
-    m_thumbnails->AddCustomExtention(wxT("js"));
-    m_thumbnails->AddCustomExtention(wxT("ass"));
-    m_thumbnails->SetFilter(wxTHUMB_FILTER_IMAGES|wxTHUMB_FILTER_VIDEOS|wxTHUMB_FILTER_AUDIOS|wxTHUMB_FILTER_CUSTOM);
+    m_thumbnails->SetFilter(wxTHUMB_FILTER_IMAGES|wxTHUMB_FILTER_VIDEOS|wxTHUMB_FILTER_AUDIOS|wxTHUMB_FILTER_SUBTITLES);
 	
 	// set file browser dir
 	wxString fbDir = s_config.GetFileBrowserDir();
@@ -219,6 +211,13 @@ MainWin::MainWin(): wxFrame(NULL, -1, _T(""), wxDefaultPosition, wxDefaultSize,
     wxMenu* thumbGlobalMenu = new wxMenu;
     thumbGlobalMenu->Append(REFRESH_ID, _("&Refresh") + wxString(wxT("\tF5")));
     m_thumbnails->SetGlobalPopupMenu(thumbGlobalMenu);
+    
+    m_fileHistory.Load(*s_config.GetConfigBase());
+//    m_fileHistory.UseMenu()
+}
+
+MainWin::~MainWin() {
+	m_cache.Clear();
 }
 
 void MainWin::set_properties() {
@@ -241,14 +240,14 @@ void MainWin::set_properties() {
 #endif
 	
 	// tool bar
-	m_toolbar->AddTool(wxID_NEW,  wxT(""), ICON_NEW,  wxNullBitmap,
-	  wxITEM_NORMAL, _("New"), wxT(""));
-	m_toolbar->AddTool(wxID_OPEN, wxT(""), ICON_OPEN, wxNullBitmap,
-	  wxITEM_NORMAL, _("Open..."), wxT(""));
-	m_toolbar->AddTool(wxID_SAVE, wxT(""), ICON_SAVE, wxNullBitmap,
-	  wxITEM_NORMAL, _("Save"), wxT(""));
-	m_toolbar->AddTool(MENU_BURN_ID,  wxT(""), wxBITMAP_FROM_MEMORY(run),  wxNullBitmap,
-	  wxITEM_NORMAL, _("Burn..."), wxT(""));
+	m_toolbar->AddTool(wxID_NEW,  wxT(""), ICON_NEW,  wxNullBitmap, wxITEM_NORMAL, _("New"), wxT(""));
+	m_toolbar->AddTool(wxID_OPEN, wxT(""), ICON_OPEN, wxNullBitmap, wxITEM_NORMAL, _("Open..."), wxT(""));
+	m_toolbar->AddTool(wxID_SAVE, wxT(""), ICON_SAVE, wxNullBitmap, wxITEM_NORMAL, _("Save"), wxT(""));
+	m_toolbar->AddTool(MENU_BURN_ID,  wxT(""), wxBITMAP_FROM_MEMORY(run), wxNullBitmap, wxITEM_NORMAL,
+			_("Burn..."), wxT(""));
+	m_toolbar->AddSeparator();
+	m_toolbar->AddTool(MENU_DVD_ADD_FILE_ID,  wxT(""), wxBITMAP_FROM_MEMORY(add), wxNullBitmap, wxITEM_NORMAL,
+			_("Add File..."), wxT(""));
 	m_toolbar->Realize();
 	
 	// dir tree
@@ -299,131 +298,110 @@ void MainWin::do_layout() {
 	m_splitterTitleSash = 0;
 }
 
-void MainWin::ShowDirTree(bool show)
-{
-  m_dirBar->ToggleTool(DIR_TREE_BT_ID, show);
-  if (show)
-  {
-    if (!m_splitterDir->IsSplit())
-    {
-      m_splitterDir->SplitVertically(m_dirTree, m_splitterThumb, m_splitterDirSash);
-      m_dirTree->Show(true);
-    }
-  }
-  else
-  {
-    if (m_splitterDir->IsSplit())
-    {
-      m_splitterDirSash = m_splitterDir->GetSashPosition();
-      
-      // animate unsplitting
-      m_dirTree->Freeze();
-      wxStopWatch sw;
-      float step = 0;
-      int pos = m_splitterDirSash - 10;
-      while (pos > 0)
-      {
-        m_splitterDir->SetSashPosition(pos);
-        while (wxTheApp->Pending())
-          wxTheApp->Dispatch();
-        if (step == 0)
-          step = sw.Time();
-        pos -= int(step);
-      }
-      m_splitterDir->Unsplit(m_dirTree);
-      m_dirTree->SetSize(0,0);
-      m_dirTree->Thaw();
-    }
-  }
-}
+void MainWin::ShowDirTree(bool show) {
+	m_dirBar->ToggleTool(DIR_TREE_BT_ID, show);
+	if (show && !m_splitterDir->IsSplit()) {
+		m_splitterDir->SplitVertically(m_dirTree, m_splitterThumb, m_splitterDirSash);
+		m_dirTree->Show(true);
+	} else if (!show && m_splitterDir->IsSplit()) {
+		m_splitterDirSash = m_splitterDir->GetSashPosition();
 
-void MainWin::OnDirTreeBt(wxCommandEvent& event)
-{
-  if (!m_splitterDir->IsSplit())
-  {
-	ShowDirTree(true);
-	wxTreeEvent evt;
-	OnDirSelected(evt);
-  }
-  else
-	ShowDirTree(false);
-}
-
-void MainWin::OnBackgroundsBt(wxCommandEvent& event)
-{
-  ShowDirTree(false);
-  m_thumbnails->ShowFileNames(false);
-  m_thumbnails->ShowDir(BACKGROUNDS_DIR);
-  m_thumbnails->SetCaption(_("Backgrounds"));
-}
-
-void MainWin::OnButtonsBt(wxCommandEvent& event)
-{
-  wxArrayString buttons;
-  wxTextFile file;
-  if (file.Open(BUTTONS_DIR + wxT("buttons.lst")))
-  {
-    for (int i=0; i<(int)file.GetLineCount(); i++)
-    {
-      wxFileName fileName(file[i]);
-      if (fileName.IsRelative())
-        fileName.MakeAbsolute(BUTTONS_DIR);
-      if (wxFile::Exists(fileName.GetFullPath()))
-        buttons.Add(fileName.GetFullPath());
-    }
-  }
-  wxString fname = wxFindFirstFile(BUTTONS_DIR + _T("*.xml"));
-  while (!fname.IsEmpty())
-  {
-	if (buttons.Index(fname) == -1)
-      buttons.Add(fname);
-    fname = wxFindNextFile();
-  }
-  
-  ShowDirTree(false);
-  m_thumbnails->Clear();
-  Menu menu;
-  for (int i=0; i<(int)buttons.Count(); i++)
-  {
-	wxThumb* thumb = new wxThumb(buttons[i]);
-	wxLogNull log;
-	MenuObject bt(&menu, buttons[i], 0, 0, _("button"));
-	thumb->SetImage(bt.GetImage(m_thumbnails->GetThumbWidth(),
-	  m_thumbnails->GetThumbHeight()));
-    m_thumbnails->InsertItem(thumb);
-  }
-  m_thumbnails->SetCaption(_("Buttons"));
-}
-
-void MainWin::OnDirSelected(wxTreeEvent& event)
-{
-  if (!m_dvd)
-    return;
-  m_thumbnails->ShowFileNames();
-  m_thumbnails->ShowDir(m_dirTree->GetPath());
-  s_config.SetFileBrowserLastDir(m_dirTree->GetPath());
-}
-
-void MainWin::OnThumbInfo(wxCommandEvent& event)
-{
-  wxString text;
-  if (m_thumbnails->GetPointed() != -1)
-  {
-	if (m_dirBar->GetToolState(BACKGROUNS_BT_ID))
-		text = _("Double click to set the background.");
-	else if (m_dirBar->GetToolState(BUTTONS_BT_ID))
-		text = m_thumbnails->GetPointedItem()->GetFilename().AfterLast(wxFILE_SEP_PATH);
-	else {
-		double size = wxFile(m_thumbnails->GetPointedItem()->GetFilename()).Length()/1024;
-		if (size < 1024)
-			text.Printf(_("%s (%.1f KB)"),m_thumbnails->GetPointedItem()->GetFilename().c_str(), size);
-		else if (size < 1024*1024)
-			text.Printf(_("%s (%.1f MB)"),m_thumbnails->GetPointedItem()->GetFilename().c_str(), size/1024);
-		else
-			text.Printf(_("%s (%.1f GB)"),m_thumbnails->GetPointedItem()->GetFilename().c_str(), size/1024/1024);
+		// animate unsplitting
+		m_dirTree->Freeze();
+		wxStopWatch sw;
+		float step = 0;
+		int pos = m_splitterDirSash - 10;
+		while (pos > 0) {
+			m_splitterDir->SetSashPosition(pos);
+			while (wxTheApp->Pending())
+				wxTheApp->Dispatch();
+			if (step == 0)
+				step = sw.Time();
+			pos -= int(step);
+		}
+		m_splitterDir->Unsplit(m_dirTree);
+		m_dirTree->SetSize(0, 0);
+		m_dirTree->Thaw();
 	}
-  }
-  m_statusbar->SetStatusText(text, 0);
+}
+
+void MainWin::OnDirTreeBt(wxCommandEvent& event) {
+	if (!m_splitterDir->IsSplit()) {
+		ShowDirTree(true);
+		wxTreeEvent evt;
+		OnDirSelected(evt);
+	} else
+		ShowDirTree(false);
+}
+
+void MainWin::OnBackgroundsBt(wxCommandEvent& event) {
+	ShowDirTree(false);
+	m_thumbnails->ShowFileNames(false);
+	m_thumbnails->ShowDir(BACKGROUNDS_DIR);
+	m_thumbnails->SetCaption(_("Backgrounds"));
+}
+
+void MainWin::OnButtonsBt(wxCommandEvent& event) {
+	wxArrayString buttons;
+	wxTextFile file;
+	if (file.Open(BUTTONS_DIR+ wxT("buttons.lst"))) {
+		for (int i=0; i<(int)file.GetLineCount(); i++) {
+			wxFileName fileName(file[i]);
+			if (fileName.IsRelative())
+			fileName.MakeAbsolute(BUTTONS_DIR);
+			if (wxFile::Exists(fileName.GetFullPath()))
+			buttons.Add(fileName.GetFullPath());
+		}
+	}
+	wxString fname = wxFindFirstFile(BUTTONS_DIR + _T("*.xml"));
+	while (!fname.IsEmpty()) {
+		if (buttons.Index(fname) == -1)
+		buttons.Add(fname);
+		fname = wxFindNextFile();
+	}
+
+	ShowDirTree(false);
+	m_thumbnails->Clear();
+	Menu menu;
+	for (int i=0; i<(int)buttons.Count(); i++) {
+		wxThumb* thumb = new wxThumb(buttons[i]);
+		wxLogNull log;
+		MenuObject bt(&menu, buttons[i], 0, 0, _("button"));
+		thumb->SetImage(bt.GetImage(m_thumbnails->GetThumbWidth(),
+						m_thumbnails->GetThumbHeight()));
+		m_thumbnails->InsertItem(thumb);
+	}
+	m_thumbnails->SetCaption(_("Buttons"));
+}
+
+void MainWin::OnDirSelected(wxTreeEvent& event) {
+	if (!m_dvd)
+		return;
+	m_thumbnails->ShowFileNames();
+	m_thumbnails->ShowDir(m_dirTree->GetPath());
+	s_config.SetFileBrowserLastDir(m_dirTree->GetPath());
+}
+
+void MainWin::OnThumbInfo(wxCommandEvent& event) {
+	wxString text;
+	if (m_thumbnails->GetPointed() != -1) {
+		if (m_dirBar->GetToolState(BACKGROUNS_BT_ID)) {
+			text = _("Double click to set the background.");
+		} else if (m_dirBar->GetToolState(BUTTONS_BT_ID)) {
+			text = m_thumbnails->GetPointedItem()->GetFilename().AfterLast(wxFILE_SEP_PATH);
+		} else {
+			double size = wxFile(m_thumbnails->GetPointedItem()->GetFilename()).Length() / 1024;
+			if (size < 1024)
+				text.Printf(_("%s (%.1f KB)"),m_thumbnails->GetPointedItem()->GetFilename().c_str(), size);
+			else if (size < 1024 * 1024)
+				text.Printf(_("%s (%.1f MB)"),m_thumbnails->GetPointedItem()->GetFilename().c_str(),
+						size / 1024);
+			else
+				text.Printf(_("%s (%.1f GB)"),m_thumbnails->GetPointedItem()->GetFilename().c_str(),
+						size / 1024 / 1024);
+		}
+	}
+	m_statusbar->SetStatusText(text, 0);
 }
 
 void MainWin::OnTitleInfo(wxCommandEvent& event) {
@@ -452,77 +430,64 @@ void MainWin::OnMenuObjectInfo(wxCommandEvent& event) {
 	wxString text;
 	wxString objId = m_menuEditor->GetPointed();
 	if (objId.length()) {
-		text = objId;
 		MenuObject* obj = m_menuEditor->GetMenu()->GetObject(objId);
-		text = obj->GetId();
-		if (obj->IsButton()) {
-			long l = 0;
-			text.Mid(6).ToLong(&l);
-			text = _("button") + wxString::Format(wxT(" %d: "), l) + obj->GetAction(true);
+		if (obj) {
+			text = obj->GetId(true);
+			if (obj->IsButton())
+				text += wxT(": ") + obj->GetAction(true);
 		} else
-			text = _("object") + wxString(wxT(" ")) + text.Mid(3);
+			text = objId;
 	}
 	m_statusbar->SetStatusText(text, 0);
 }
 
-void MainWin::OnThumbDoubleClick(wxCommandEvent& event)
-{
-  if (m_thumbnails->GetSelected() == -1)
-    return;
-  wxString filename = m_thumbnails->GetSelectedItem()->GetFilename();
-  if (wxImage::FindHandler(filename.AfterLast(wxT('.')).Lower(), -1) ||
-      MPEG::IsValid(filename))
-    m_menuEditor->SetBackground(filename);
+void MainWin::OnThumbDoubleClick(wxCommandEvent& event) {
+	if (m_thumbnails->GetSelected() == -1)
+		return;
+	wxString filename = m_thumbnails->GetSelectedItem()->GetFilename();
+	if (wxImage::FindHandler(filename.AfterLast(wxT('.')).Lower(), -1) || MPEG::IsValid(filename))
+		m_menuEditor->SetBackground(filename);
 }
 
-void MainWin::OnSetBackground(wxCommandEvent& event)
-{
-  wxString filename = m_thumbnails->GetSelectedItem()->GetFilename();
-  m_menuEditor->SetBackground(filename);
+void MainWin::OnSetBackground(wxCommandEvent& event) {
+	wxString filename = m_thumbnails->GetSelectedItem()->GetFilename();
+	m_menuEditor->SetBackground(filename);
 }
 
 void MainWin::OnRefresh(wxCommandEvent& event) {
 	m_thumbnails->UpdateItems();
 }
 
-void MainWin::OnUpdateUISetBackground(wxUpdateUIEvent& event)
-{
-  if (m_thumbnails->GetSelected() == -1)
-    event.Enable(false);
-  else
-  {
-    wxString filename = m_thumbnails->GetSelectedItem()->GetFilename();
-    event.Enable(
-      wxImage::FindHandler(filename.AfterLast(wxT('.')).Lower(), -1) ||
-      wxThumbnails::IsVideo(filename));
-  }
+void MainWin::OnUpdateUISetBackground(wxUpdateUIEvent& event) {
+	if (m_thumbnails->GetSelected() == -1)
+		event.Enable(false);
+	else {
+		wxString filename = m_thumbnails->GetSelectedItem()->GetFilename();
+		event.Enable(wxImage::FindHandler(filename.AfterLast(wxT('.')).Lower(), -1) || wxThumbnails::IsVideo(filename));
+	}
 }
 
-void MainWin::OnSplitterTitleSash(wxSplitterEvent &event)
-{
-  int width, height;
-  GetClientSize(&width, &height);
-  m_splitterTitleSash = height - m_splitterTitle->GetSashPosition();
-}
-
-void MainWin::OnResize(wxSizeEvent &event)
-{
-  wxFrame::OnSize(event);
-  if (m_splitterTitle)
-  {
-    int width, height;
+void MainWin::OnSplitterTitleSash(wxSplitterEvent &event) {
+	int width, height;
 	GetClientSize(&width, &height);
-	m_splitterTitle->SetSashPosition(height - m_splitterTitleSash);
-  }
+	m_splitterTitleSash = height - m_splitterTitle->GetSashPosition();
 }
 
-void MainWin::OnClose(wxCloseEvent &event)
-{
-  if (!TestSaved())
-	return;
-  if (!IsIconized())
-	  s_config.SetMainWinLocation(GetRect(), IsMaximized());
-  Destroy();
+void MainWin::OnResize(wxSizeEvent &event) {
+	wxFrame::OnSize(event);
+	if (m_splitterTitle) {
+		int width, height;
+		GetClientSize(&width, &height);
+		m_splitterTitle->SetSashPosition(height - m_splitterTitleSash);
+	}
+}
+
+void MainWin::OnClose(wxCloseEvent &event) {
+	if (!TestSaved())
+		return;
+	if (!IsIconized())
+		s_config.SetMainWinLocation(GetRect(), IsMaximized());
+	Destroy();
 }
 
 void MainWin::NewDVD(wxString discLabel, DiscCapacity capacity, int videoBitrate,
@@ -538,78 +503,78 @@ void MainWin::NewDVD(wxString discLabel, DiscCapacity capacity, int videoBitrate
 	m_dvd->SetAudioFormat(audioFormat);
 	m_dvd->SetAspectRatio(aspectRatio);
 	m_dvd->AddTitleset();
-	m_dvd->AddMenu(0);
+	m_dvd->AddMenu(-1);
 	m_filename = _T("");
 	UpdateDVD();
 	SetChanged(false);
+	m_cache.Clear();
 }
 
-void MainWin::Open(wxString filename)
-{
-  DVD* dvd = new DVD;
-  if (dvd->Open(filename))
-  {
-    delete m_dvd;
-    m_dvd = dvd;
-    m_filename = filename;
-    m_lastDir = wxPathOnly(m_filename);
-    s_config.SetLastProjectDir(m_lastDir);
-	UpdateDVD();
-	SetChanged(false);
-  }
-  else
-  {
-	delete dvd;
-    wxLogError(_("Error opening '%s'"), filename.c_str());
-  }
+void MainWin::Open(wxString filename) {
+	DVD* dvd = new DVD;
+	if (dvd->Open(filename)) {
+		delete m_dvd;
+		m_dvd = dvd;
+		m_filename = filename;
+		m_lastDir = wxPathOnly(m_filename);
+		s_config.SetLastProjectDir(m_lastDir);
+		UpdateDVD();
+		SetChanged(false);
+		m_cache.Clear();
+		m_fileHistory.AddFileToHistory(m_filename);
+		m_fileHistory.Save(*s_config.GetConfigBase());
+	} else {
+		delete dvd;
+		wxLogError(_("Error opening '%s'"), filename.c_str());
+	}
 }
 
-void MainWin::UpdateDVD()
-{
-  m_titlesetManager->SetDVD(m_dvd);
-  m_titlesetManager->UpdateItems();
-  m_titlesetManager->SelectFirstMenu();
-  wxCommandEvent evt;
-  OnChangedDVD(evt);
+void MainWin::ShowOpenDlg() {
+	wxFileDialog dlg(this, _("Open a DVDStyler project file"), m_lastDir, _T(""), _T("*.xml"), wxOPEN);
+	dlg.Centre();
+	if (dlg.ShowModal() == wxID_OK)
+		Open(dlg.GetPath());
 }
 
-
-void MainWin::SetChanged(bool changed)
-{
-  m_changed = changed;
-  m_toolbar->EnableTool(wxID_SAVE, changed);
-  m_menubar->GetMenu(0)->Enable(wxID_SAVE, changed);
-  wxString fname = m_filename;
-  if (!fname.length())
-	fname = _("Untitled");
-  SetTitle(_T("DVDStyler - ") + fname +
-    (m_changed || !m_filename.length() ? _T("*") : _T("")));
+void MainWin::UpdateDVD() {
+	m_titlesetManager->SetDVD(m_dvd);
+	m_titlesetManager->UpdateItems();
+	m_titlesetManager->SelectFirstMenu();
+	wxCommandEvent evt;
+	OnChangedDVD(evt);
 }
 
-void MainWin::OnChangedMenu(wxCommandEvent& event)
-{
-  SetChanged(true);
-  if (m_titlesetManager->GetSelectedMenu()<0)
-	return;
-  m_titlesetManager->GetItem(m_titlesetManager->GetSelectedMenu())->Update();
-  m_titlesetManager->Refresh();
+void MainWin::SetChanged(bool changed) {
+	m_changed = changed;
+	m_toolbar->EnableTool(wxID_SAVE, changed);
+	m_menubar->GetMenu(0)->Enable(wxID_SAVE, changed);
+	wxString fname = m_filename;
+	if (!fname.length())
+		fname = _("Untitled");
+	SetTitle(_T("DVDStyler - ") + fname + (m_changed || !m_filename.length() ? _T("*") : _T("")));
 }
 
-void MainWin::OnSelectMenu(wxCommandEvent& event)
-{
-  bool changed = GetChanged();
-  int tsi = 0;
-  int pgci = 0;
-  if (m_titlesetManager->GetSelectedMenu()>=0)
-  {
-    int id = m_titlesetManager->GetSelectedMenuId();
-    tsi = DVD::GetTsi(id);
-    pgci = DVD::GetPgci(id);
-  }
-  Menu* menu = m_dvd->GetMenu(tsi, pgci);
-  m_menuEditor->SetMenu(m_dvd, menu, tsi, pgci);
-  m_menuEditor->Update();
-  SetChanged(changed);
+void MainWin::OnChangedMenu(wxCommandEvent& event) {
+	SetChanged(true);
+	if (m_titlesetManager->GetSelectedMenu() < 0)
+		return;
+	m_titlesetManager->GetItem(m_titlesetManager->GetSelectedMenu())->Update();
+	m_titlesetManager->Refresh();
+}
+
+void MainWin::OnSelectMenu(wxCommandEvent& event) {
+	bool changed = GetChanged();
+	int tsi = 0;
+	int pgci = 0;
+	if (m_titlesetManager->GetSelectedMenu() >= 0) {
+		int id = m_titlesetManager->GetSelectedMenuId();
+		tsi = DVD::GetTsi(id);
+		pgci = DVD::GetPgci(id);
+	}
+	Menu* menu = m_dvd->GetMenu(tsi, pgci);
+	m_menuEditor->SetMenu(m_dvd, menu, tsi, pgci);
+	m_menuEditor->Update();
+	SetChanged(changed);
 }
 
 void MainWin::OnChangedDVD(wxCommandEvent& event) {
@@ -639,57 +604,45 @@ bool MainWin::TestSaved() {
 	return true;
 }
 
-void MainWin::OnNew(wxCommandEvent& event)
-{
-  if (!TestSaved())
-    return;
-  NewProjectDlg dlg(this);
-  if (dlg.ShowModal() == wxID_OK)
-	NewDVD(dlg.GetLabel(), dlg.GetCapacity(), dlg.GetVideoBitrate(),
-			dlg.GetVideoFormat(), dlg.GetAudioFormat(), dlg.GetAspectRatio());
-}
+// wxGlade: add MainWin event handlers
 
-void MainWin::OnOpen(wxCommandEvent& event)
-{
-  if (!TestSaved())
-	return;
-  wxFileDialog dlg( this, _("Open a DVDStyler project file"), m_lastDir,
-	_T(""), _T("*.xml"), wxOPEN);
-  dlg.Centre();
-  if (dlg.ShowModal() == wxID_OK)
-	Open(dlg.GetPath());
-}
-
-void MainWin::OnSave(wxCommandEvent& event)
-{
-  if (m_filename.length())
-  {
-	if (m_dvd->Save(m_filename))
-	  SetChanged(false);
-  }
-  else
-	OnSaveAs(event);
-}
-
-void MainWin::OnSaveAs(wxCommandEvent& event)
-{
-  if (m_dvd->GetTitlesets().Count() ||
-      m_dvd->GetVmgm().Count())
-  {
-	wxFileDialog dlg(this, _("Save a DVDStyler project file"), m_lastDir,
-	  _T("dvd.xml"), _T("*.xml"), wxSAVE | wxOVERWRITE_PROMPT);
-	dlg.Centre();
+void MainWin::OnNew(wxCommandEvent& event) {
+	if (!TestSaved())
+		return;
+	NewProjectDlg dlg(this);
 	if (dlg.ShowModal() == wxID_OK)
-	{
-	  m_filename = dlg.GetPath();
-	  m_lastDir = wxPathOnly(m_filename);
-	  s_config.SetLastProjectDir(m_lastDir);
-	  if (m_dvd->Save(m_filename))
-		SetChanged(false);
-	}
-  }
-  else
-	wxLogError(_("Nothing to save"));
+		NewDVD(dlg.GetLabel(), dlg.GetCapacity(), dlg.GetVideoBitrate(), dlg.GetVideoFormat(),
+				dlg.GetAudioFormat(), dlg.GetAspectRatio());
+}
+
+void MainWin::OnOpen(wxCommandEvent& event) {
+	if (!TestSaved())
+		return;
+	ShowOpenDlg();
+}
+
+void MainWin::OnSave(wxCommandEvent& event) {
+	if (m_filename.length()) {
+		if (m_dvd->Save(m_filename))
+			SetChanged(false);
+	} else
+		OnSaveAs(event);
+}
+
+void MainWin::OnSaveAs(wxCommandEvent& event) {
+	if (m_dvd->GetTitlesets().Count() || m_dvd->GetVmgm().Count()) {
+		wxFileDialog dlg(this, _("Save a DVDStyler project file"), m_lastDir, _T("dvd.xml"), _T("*.xml"),
+				wxSAVE | wxOVERWRITE_PROMPT);
+		dlg.Centre();
+		if (dlg.ShowModal() == wxID_OK) {
+			m_filename = dlg.GetPath();
+			m_lastDir = wxPathOnly(m_filename);
+			s_config.SetLastProjectDir(m_lastDir);
+			if (m_dvd->Save(m_filename))
+				SetChanged(false);
+		}
+	} else
+		wxLogError(_("Nothing to save"));
 }
 /**
  * Starts compiling without closing
@@ -737,44 +690,63 @@ please drag a video file into the bottom section of the window."), emptyTsi+1),
 		return;
 	}
 	// show burn dialog ()
-	BurnDlg burnDlg(this);
+	BurnDlg burnDlg(this, m_dvd, &m_cache);
 	if (!autoStart && burnDlg.ShowModal() != wxID_OK)
 		return;
 	// Progress dialog
-	ProgressDlg progressDlg(this, autoStart);
-	progressDlg.Start(&burnDlg, m_dvd);
-	if (!autoStart) {
+	ProgressDlg progressDlg(this, &m_cache, autoStart);
+	bool ok = progressDlg.Start(&burnDlg, m_dvd);
+	if (autoStart) {
+		Close(true);
+	} else if (ok) {
 		DonateDlg donateDlg(this);
 		donateDlg.ShowModal();
-	} else
-		Close(true);
+	}
 }
 
-void MainWin::OnExit(wxCommandEvent& event)
-{
-  Close(true);
+void MainWin::OnExit(wxCommandEvent& event) {
+	Close(true);
 }
 
-void MainWin::OnAddMenu(wxCommandEvent& WXUNUSED(event))
-{
-  m_titlesetManager->SetSelected(-1);
-  m_titlesetManager->AddMenu();
+void MainWin::OnAddFile(wxCommandEvent& event) {
+	wxString video = wxThumbnails::GetVideoExtWildcard();
+	wxString audio = wxThumbnails::GetAudioExtWildcard();
+	wxString subtitle = wxThumbnails::GetSubtitleExtWildcard();
+	wxFileDialog dialog(this, _("Choose a file"), wxT(""), wxT(""),
+			_("Audio/Video/Subtitle Files") + wxString::Format(wxT("|%s%s%s|"), video.c_str(), audio.c_str(), subtitle.c_str())
+			+ _("Audio Files") + wxString::Format(wxT(" (%s)|%s|"), audio.c_str(), audio.c_str())
+			+ _("Video Files") + wxString::Format(wxT(" (%s)|%s|"), video.c_str(), video.c_str())
+			+ _("Subtitle Files") + wxString::Format(wxT(" (%s)|%s|"), subtitle.c_str(), subtitle.c_str()),
+			wxFD_OPEN);
+	// set file browser dir
+	dialog.SetDirectory(s_config.GetLastAddDir() + wxFILE_SEP_PATH);
+	if (dialog.ShowModal() == wxID_OK) {
+		if (wxThumbnails::IsVideo(dialog.GetPath()))
+			m_titlesetManager->AddVideo(dialog.GetPath());
+		else if (wxThumbnails::IsAudio(dialog.GetPath()))
+			m_titlesetManager->AddAudio(dialog.GetPath());
+		else
+			m_titlesetManager->AddSubtitles(dialog.GetPath());
+		s_config.SetLastAddDir(dialog.GetDirectory());
+	}
 }
 
-void MainWin::OnAddVmMenu(wxCommandEvent& WXUNUSED(event))
-{
-  m_titlesetManager->AddVmMenu();
+void MainWin::OnAddMenu(wxCommandEvent& event) {
+	m_titlesetManager->SetSelected(-1);
+	m_titlesetManager->AddMenu();
 }
 
-void MainWin::OnAddTitleset(wxCommandEvent& WXUNUSED(event))
-{
-  m_titlesetManager->AddTitleset();
+void MainWin::OnAddVmMenu(wxCommandEvent& event) {
+	m_titlesetManager->AddVmMenu();
 }
 
-void MainWin::OnAddTitlesetUpdateUI(wxUpdateUIEvent& event)
-{
-  int cnt = m_dvd->GetTitlesets().Count();
-  event.Enable(cnt>0 &&	m_dvd->GetTitlesets()[cnt-1]->GetTitles().Count());
+void MainWin::OnAddTitleset(wxCommandEvent& event) {
+	m_titlesetManager->AddTitleset();
+}
+
+void MainWin::OnAddTitlesetUpdateUI(wxUpdateUIEvent& event) {
+	int cnt = m_dvd->GetTitlesets().Count();
+	event.Enable(cnt > 0 && m_dvd->GetTitlesets()[cnt - 1]->GetTitles().Count());
 }
 
 void MainWin::OnDVDOptions(wxCommandEvent& event) {
@@ -792,12 +764,10 @@ void MainWin::OnSettings(wxCommandEvent& event) {
 	dlg.ShowModal();
 }
 
-void MainWin::OnHelpContents(wxCommandEvent& WXUNUSED(event)) {
+void MainWin::OnHelpContents(wxCommandEvent& event) {
 	m_helpController.DisplayContents();
 }
 
-void MainWin::OnAbout(wxCommandEvent& WXUNUSED(event)) {
+void MainWin::OnAbout(wxCommandEvent& event) {
 	About(this);
 }
-
-// wxGlade: add MainWin event handlers
