@@ -3,7 +3,7 @@
 // Purpose:     The class to store a DVD Menu
 // Author:      Alex Thuering
 // Created:	04.11.2003
-// RCS-ID:      $Id: Menu.cpp,v 1.68 2009/01/01 17:15:38 ntalex Exp $
+// RCS-ID:      $Id: Menu.cpp,v 1.71 2009/08/03 21:27:23 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -11,6 +11,7 @@
 #include "Menu.h"
 #include "MenuPalettes.h"
 #include "Palette3D.h"
+#include "DVD.h"
 #include <wxVillaLib/SConv.h>
 #include <wxVillaLib/utils.h>
 #include <wxSVG/svg.h>
@@ -527,7 +528,7 @@ bool Menu::SaveSpumux(wxString fileName, wxString btFile, wxString hlFile, wxStr
 	return xml.Save(fileName);
 }
 
-wxSvgXmlNode* Menu::GetXML(DVDFileType type, int playAllRegister, wxSvgXmlNode* node) {
+wxSvgXmlNode* Menu::GetXML(DVDFileType type, DVD* dvd, wxSvgXmlNode* node) {
 	if (node == NULL)
 		node = new wxSvgXmlNode(wxSVGXML_ELEMENT_NODE, _T("menu"));
   
@@ -549,6 +550,35 @@ wxSvgXmlNode* Menu::GetXML(DVDFileType type, int playAllRegister, wxSvgXmlNode*
 				if (bgFile.StartsWith(dir))
 	 				SetBackground(svg, bgFile.substr(dir.length()));
 			}
+			// set relative path
+			wxSVGElement* elem = svg;
+			while (elem != NULL && dvd != NULL && type == DVDSTYLER_XML) {
+				if (elem->GetDtd() == wxSVG_IMAGE_ELEMENT) {
+					wxSVGImageElement* imgElem = (wxSVGImageElement*)elem;
+					wxString href = imgElem->GetHref();
+					wxString frameStr;
+					if (href.Find(wxT('#')) > 0) {
+						frameStr = wxT('#') + href.AfterLast(wxT('#'));
+						href = href.BeforeLast(wxT('#'));
+					}
+					wxFileName fname(href);
+					if (fname.GetPath() == dvd->GetPath(false))
+						imgElem->SetHref(fname.GetFullName() + frameStr);
+					else if (fname.GetPath().StartsWith(dvd->GetPath(false)))
+						imgElem->SetHref(fname.GetPath().substr(dvd->GetPath(false).length() + 1)
+								+ wxFILE_SEP_PATH + fname.GetFullName() + frameStr);
+				}
+				if (elem->GetChildren() != NULL && (elem->GetDtd() == wxSVG_SVG_ELEMENT
+						|| elem->GetDtd() == wxSVG_DEFS_ELEMENT || elem->GetDtd() == wxSVG_G_ELEMENT)) {
+					elem = (wxSVGElement*)elem->GetChildren();
+				} else {
+					while (elem != NULL && elem->GetNextSibling() == NULL)
+						elem = (wxSVGElement*) elem->GetParent();
+					if (elem != NULL)
+						elem = (wxSVGElement*) elem->GetNextSibling();
+				}
+			}
+			// append node
 			node->AppendChild(svg);
 		}
 	}
@@ -557,7 +587,7 @@ wxSvgXmlNode* Menu::GetXML(DVDFileType type, int playAllRegister, wxSvgXmlNode*
 	for (int i=0; i<GetObjectsCount(); i++) {
 		MenuObject* obj = GetObject(i);
 		if (type == DVDSTYLER_XML || obj->IsButton())
-			node->AddChild(obj->GetXML(type, false, playAllRegister));
+			node->AddChild(obj->GetXML(type, false, dvd != NULL ? dvd->GetPlayAllRegister() : -1));
 	}
 	
 	// actions
@@ -567,7 +597,7 @@ wxSvgXmlNode* Menu::GetXML(DVDFileType type, int playAllRegister, wxSvgXmlNode*
 	return node;
 }
 
-bool Menu::PutXML(wxSvgXmlNode* node) {
+bool Menu::PutXML(wxSvgXmlNode* node, DVD* dvd) {
 	if (node->GetName() == _T("spumux"))
 		node = node->GetChildren();
 	if (node != NULL && node->GetName() == _T("menu")) {
@@ -592,8 +622,43 @@ bool Menu::PutXML(wxSvgXmlNode* node) {
 			// fix path
 			if (GetBackground().length() > 0) {
 				wxFileName fn(GetBackground());
-				if (fn.IsRelative())
-					SetBackground(BACKGROUNDS_DIR + GetBackground());
+				if (fn.IsRelative()) {
+					if (wxFileExists(dvd->GetPath() + fn.GetFullPath()))
+						SetBackground(dvd->GetPath() + fn.GetFullPath());
+					else if (wxFileExists(dvd->GetPath() + fn.GetFullName()))
+						SetBackground(dvd->GetPath() + fn.GetFullName());
+					else if (BACKGROUNDS_DIR + fn.GetFullName())
+						SetBackground(BACKGROUNDS_DIR + fn.GetFullName());
+					else
+						SetBackground(dvd->GetPath() + fn.GetFullPath());
+				}
+			}
+			// fix relative path
+			wxSVGElement* elem = m_svg->GetRootElement();
+			while (elem != NULL && dvd != NULL) {
+				if (elem->GetDtd() == wxSVG_IMAGE_ELEMENT) {
+					wxSVGImageElement* imgElem = (wxSVGImageElement*)elem;
+					wxString href = imgElem->GetHref();
+					wxString frameStr;
+					if (href.Find(wxT('#')) > 0) {
+						frameStr = wxT('#') + href.AfterLast(wxT('#'));
+						href = href.BeforeLast(wxT('#'));
+					}
+					wxFileName fname(href);
+					if (fname.IsRelative())
+						imgElem->SetHref(dvd->GetPath() + fname.GetFullPath() + frameStr);
+					else if (!wxFileExists(href) && wxFileExists(dvd->GetPath() + fname.GetFullName()))
+						imgElem->SetHref(dvd->GetPath() + fname.GetFullName() + frameStr);
+				}
+				if (elem->GetChildren() != NULL && (elem->GetDtd() == wxSVG_SVG_ELEMENT
+						|| elem->GetDtd() == wxSVG_DEFS_ELEMENT || elem->GetDtd() == wxSVG_G_ELEMENT)) {
+					elem = (wxSVGElement*)elem->GetChildren();
+				} else {
+					while (elem != NULL && elem->GetNextSibling() == NULL)
+						elem = (wxSVGElement*) elem->GetParent();
+					if (elem != NULL)
+						elem = (wxSVGElement*) elem->GetNextSibling();
+				}
 			}
 	    } else { // deprecated
 	    	if (node->GetPropVal(wxT("bgFile"), &val))
