@@ -3,7 +3,7 @@
 // Purpose:     The widget to edit a DVD Menu
 // Author:      Alex Thuering
 // Created:     11.10.2003
-// RCS-ID:      $Id: MenuEditor.cpp,v 1.24 2009/02/11 17:48:35 ntalex Exp $
+// RCS-ID:      $Id: MenuEditor.cpp,v 1.27 2009/06/14 16:56:12 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -14,6 +14,7 @@
 #include "MPEG.h"
 #include "TitlesetManager.h"
 #include "Utils.h"
+#include "Config.h"
 #include <wxVillaLib/ImageProc.h>
 #include <wxVillaLib/Thumbnails.h>
 #include <wxVillaLib/utils.h>
@@ -24,6 +25,7 @@
 #include <wx/utils.h>
 #include <wx/clipbrd.h>
 #include <wx/artprov.h>
+#include <wx/filename.h>
 
 #define OBJECTS_DIR wxFindDataDirectory(_T("objects"))
 #define BUTTONS_DIR wxFindDataDirectory(_T("buttons"))
@@ -70,23 +72,27 @@ public:
 				wxLogError(filename + _(": unknown format"));
 				return false;
 			}
-			if (m_menuEditor->GetTitlesetManager()->AddVideo(filename)) {
-				int title = m_menuEditor->GetDVD()->GetPgcArray(m_menuEditor->GetTsi(), false).GetCount()-1;
+			int tsi = m_menuEditor->GetTsi();
+			if (tsi == -1)
+				tsi = m_menuEditor->GetDVD()->GetTitlesets().size() - 1;
+			if (tsi == -1)
+				tsi = 0;
+			if (m_menuEditor->GetTitlesetManager()->AddVideo(filename, true, tsi)) {
+				int title = m_menuEditor->GetDVD()->GetPgcArray(tsi, false).GetCount()-1;
 				if (objId.length() && m_menuEditor->GetMenu()->GetObject(objId)->IsButton()) {
 					MenuObject* obj = m_menuEditor->GetMenu()->GetObject(objId);
 					MenuObjectParam* initParam = obj->GetInitParam();
 					if (initParam && initParam->type == wxT("image")) {
-						Vob* vob = m_menuEditor->GetDVD()->GetVob(m_menuEditor->GetTsi(), false, title,
-								0);
+						Vob* vob = m_menuEditor->GetDVD()->GetVob(tsi, false, title, 0);
 						if (vob) {
-							obj->SetParam(initParam->name,
-									vob->GetVideoFrameUri(0));
+							obj->SetParam(initParam->name, vob->GetVideoFrameUri(0));
 							m_menuEditor->Select(objId);
 						}
 					}
+					obj->SetActionTsi(tsi);
 					obj->SetActionPgci(title*2+1);
 				} else
-					m_menuEditor->AddButton(BUTTONS_DIR + wxT("frame.xml"), x, y, title);
+					m_menuEditor->AddButton(BUTTONS_DIR + s_config.GetDefButton(), x, y, title, tsi);
 			}
 		}
 		m_menuEditor->SetFocus();
@@ -186,10 +192,8 @@ END_EVENT_TABLE()
 DEFINE_EVENT_TYPE(EVT_COMMAND_MENUEDITOR_MENU_CHANGED)
 DEFINE_EVENT_TYPE(EVT_COMMAND_MENUEDITOR_OBJECT_POINTED)
 
-MenuEditor::MenuEditor(wxWindow *parent, wxWindowID id):
-  wxSVGCtrl(parent, id),m_eventTimer
-(this, EVENT_TIMER_ID)
-{
+MenuEditor::MenuEditor(wxWindow *parent, wxWindowID id): wxSVGCtrl(parent, id),
+		m_eventTimer(this, EVENT_TIMER_ID) {
 	m_dvd = NULL;
 	m_menu = NULL;
 	DoSelect();
@@ -212,8 +216,7 @@ MenuEditor::MenuEditor(wxWindow *parent, wxWindowID id):
 	wxString fname = wxFindFirstFile(OBJECTS_DIR + _T("*.xml"));
 	int addId = MENUEDITOR_ADDMENU_OBJECT_ID;
 	Menu menu;
-	while (!fname.IsEmpty())
-	{
+	while (!fname.IsEmpty()) {
 		MenuObject obj(&menu, fname);
 		m_addMenu->Append(addId++, wxGetTranslation((const wxChar*)obj.GetTitle().GetData()));
 		m_addMenuObjects.Add(obj.GetFileName());
@@ -270,19 +273,18 @@ MenuEditor::MenuEditor(wxWindow *parent, wxWindowID id):
 	m_pmenu->Append(MENUEDITOR_PROP_ID, _("&Properties..."));
 }
 
-void MenuEditor::SetMenu(DVD* dvd, Menu* menu, int tsi, int pgci)
-{
-  m_dvd = dvd;
-  m_menu = menu;
-  m_tsi = tsi;
-  m_pgci = pgci;
-  if (m_menu) {
-    SetSVG(m_menu->GetSVG());
-    ShowSafeTV();
-    ShowGrid();
-  } else
-	Clear();
-  DoSelect();
+void MenuEditor::SetMenu(DVD* dvd, Menu* menu, int tsi, int pgci) {
+	m_dvd = dvd;
+	m_menu = menu;
+	m_tsi = tsi;
+	m_pgci = pgci;
+	if (m_menu) {
+		SetSVG(m_menu->GetSVG());
+		ShowSafeTV();
+		ShowGrid();
+	} else
+		Clear();
+	DoSelect();
 }
 
 bool MenuEditor::SetBackground(wxString fname) {
@@ -309,49 +311,43 @@ bool MenuEditor::SetBackground(wxString fname) {
 	return true;
 }
 
-void MenuEditor::SetBackgroundColour(wxColour colour)
-{
-  if (!m_menu)
-	return;
-  
-  if (m_menu->GetBackgroundColour() != colour)
-  {
-    m_menu->SetBackgroundColour(colour);
-    Update();
-  }
+void MenuEditor::SetBackgroundColour(wxColour colour) {
+	if (!m_menu)
+		return;
+
+	if (m_menu->GetBackgroundColour() != colour) {
+		m_menu->SetBackgroundColour(colour);
+		Update();
+	}
 }
 
-bool MenuEditor::AddImage(wxString fname, int x, int y)
-{
-  if (!m_menu)
-	return false;
-  if (m_menu->HasVideoBackground())
-  {
-	wxLogError(_("You can add only buttons to menu with VIDEO background"));
-	return false;
-  }
-  wxImage img;
-  if (!img.LoadFile(fname))
-    return false;
-  
-  DoSelect(m_menu->AddImage(fname, x, y), true, true);
-  return true;
+bool MenuEditor::AddImage(wxString fname, int x, int y) {
+	if (!m_menu)
+		return false;
+	if (m_menu->HasVideoBackground()) {
+		wxLogError(_("You can add only buttons to menu with VIDEO background"));
+		return false;
+	}
+	wxImage img;
+	if (!img.LoadFile(fname))
+		return false;
+
+	DoSelect(m_menu->AddImage(fname, x, y), true, true);
+	return true;
 }
 
-bool MenuEditor::AddText(wxString text, int x, int y)
-{
-  if (!m_menu)
-	return false;
-  if (m_menu->HasVideoBackground())
-  {
-	wxLogError(_("You can add only buttons to menu with VIDEO background"));
-	return false;
-  }
-  DoSelect(m_menu->AddText(text, x, y), true, true);
-  return true;
+bool MenuEditor::AddText(wxString text, int x, int y) {
+	if (!m_menu)
+		return false;
+	if (m_menu->HasVideoBackground()) {
+		wxLogError(_("You can add only buttons to menu with VIDEO background"));
+		return false;
+	}
+	DoSelect(m_menu->AddText(text, x, y), true, true);
+	return true;
 }
 
-bool MenuEditor::AddButton(wxString fname, int x, int y, int assignToTitle) {
+bool MenuEditor::AddButton(wxString fname, int x, int y, int assignToTitle, int tsi) {
 	if (!m_menu || !wxFileExists(fname))
 		return false;
 	
@@ -369,348 +365,316 @@ bool MenuEditor::AddButton(wxString fname, int x, int y, int assignToTitle) {
 	}
 
 	wxString param;
+	int title = assignToTitle >= 0 ? assignToTitle : 0;
+	tsi = tsi >= 0 ? tsi : m_tsi >= 0 ? m_tsi : 0;
 
 	Menu menu;
 	MenuObject obj(&menu, fname);
 	MenuObjectParam* initParam = obj.GetInitParam();
 	if (initParam) {
+		Vob* vob = m_dvd->GetVob(tsi, false, title, 0);
 		if (initParam->type == wxT("image")) {
-			Vob* vob = m_dvd->GetVob(m_tsi, false, assignToTitle, 0);
 			if (vob)
 				param = vob->GetVideoFrameUri(0);
-		} else
+		} else if (assignToTitle >= 0) {
+			if (vob)
+				param = wxFileName(vob->GetVideoFrameUri(0)).GetName();
+		} else {
 			param = wxString::Format(_("button%d"), i);
+		}
 	}
 
 	wxString objId = m_menu->AddObject(fname, param, x, y);
-	m_menu->GetObject(objId)->SetActionPgci(assignToTitle*2+1);
+	m_menu->GetObject(objId)->SetActionTsi(tsi);
+	m_menu->GetObject(objId)->SetActionPgci(title*2+1);
 	DoSelect(objId, true, true);
 	return true;
 }
 
-bool MenuEditor::AddObject(wxString fname, int x, int y)
-{
-  if (!m_menu || !wxFileExists(fname))
-    return false;
-  
-  wxString param;
-  
-  Menu menu;
-  MenuObject obj(&menu, fname);
-  MenuObjectParam* initParam = obj.GetInitParam();
-  if (initParam)
-  {
-    if (initParam->type == wxT("text") || initParam->type == wxT("string"))
-      param = wxGetTextFromUser(_("Please type in text to insert"),
-        _("Input text"), _T("Text"), this);
-    else if (initParam->type == wxT("image"))
-      param = wxFileSelector(_("Please choose an image to insert"),
-        wxT(""), wxGetCwd(), wxT(""),
-        _("Image Files ") + 
-#if wxCHECK_VERSION(2,6,0)
-        wxImage::GetImageExtWildcard(),
-#else
-        wxString(wxT("(*.jpg;*.bmp;*.png;*.tif;*.gif;*.pnm;*.pcx)|*.jpg;*.bmp;*.png;*.tif;*.gif;*.pnm;*.pcx")),
-#endif
-        wxOPEN|wxFILE_MUST_EXIST);
-    else
-      param = _("Text");
-    if (!param.length())
-      return false;
-  }
-  
-  DoSelect(m_menu->AddObject(fname, param, x, y), true, true);
-  return true;
-}
+bool MenuEditor::AddObject(wxString fname, int x, int y) {
+	if (!m_menu || !wxFileExists(fname))
+		return false;
 
-wxString MenuEditor::HitTest(int x, int y)
-{
-  x = (int) (x/GetScale());
-  y = (int) (y/GetScale());
-  wxString id;
-  // at frist try to find a button
-  bool button = true;
-  wxSVGUseElement* child = (wxSVGUseElement*)
-    m_doc->GetElementById(wxT("buttons"))->GetChildren();
-  while (child || button && id.length() == 0)
-  {
-  	if (!child) {
-    	// no button was found, try to find a object
-    	child = (wxSVGUseElement*) m_doc->GetElementById(wxT("objects"))->GetChildren();
-    	button = false;
-    	continue;
-    }
-    if (child->GetType() == wxSVGXML_ELEMENT_NODE &&
-        child->GetDtd() == wxSVG_USE_ELEMENT)
-    {
-      wxSVGRect bbox = child->GetBBox();
-      if (!bbox.IsEmpty() &&
-          x >= bbox.GetX() && x <= bbox.GetX() + bbox.GetWidth() &&
-          y >= bbox.GetY() && y <= bbox.GetY() + bbox.GetHeight())
-        id = child->GetId();
-    }
-    child = (wxSVGUseElement*) child->GetNext();
-  }
-  return id;
-}
+	wxString param;
 
-void MenuEditor::DoSelect(wxString id, bool refresh, bool sendEvent)
-{
-  if (!m_menu)
-    return;
-  
-  m_selected = id;
-  
-  // remove old selection
-  wxSVGRectElement* selRect =
-    (wxSVGRectElement*) m_doc->GetElementById(wxT("selection"));
-  if (selRect)
-  {
-    if (refresh)
-      RefreshRect(ScaleRect(selRect->GetBBox(), GetScale()).Inflate(4));
-    selRect->GetParent()->RemoveChild(selRect);
-  }
-  
-  // create selection rectangle
-  if (id.length())
-  {
-    wxSVGUseElement* elem = (wxSVGUseElement*) m_doc->GetElementById(id);
-    if (!elem || elem->GetDtd() != wxSVG_USE_ELEMENT)
-      return;
-    wxSVGRect bbox = elem->GetBBox();
-    
-    selRect = new wxSVGRectElement;
-    selRect->SetId(wxT("selection"));
-    selRect->SetX(bbox.GetX());
-    selRect->SetY(bbox.GetY());
-    selRect->SetWidth(bbox.GetWidth());
-    selRect->SetHeight(bbox.GetHeight());
-    selRect->SetStroke(wxSVGPaint(255, 0, 0));
-    selRect->SetFill(wxSVGPaint());
-    m_doc->GetRootElement()->AppendChild(selRect);
-    
-    if (refresh)
-      RefreshRect(ScaleRect(selRect->GetBBox(), GetScale()).Inflate(4));
-  }
-  
-  if (sendEvent)
-    SendChangedEvent();
-}
-
-void MenuEditor::OnMouseLeftButton(wxMouseEvent &event)
-{
-  SetFocus();
-  if (!m_menu)
-	return;
-  int x = event.GetX();
-  int y = event.GetY();
-  m_tt = GetTransformType(x, y);
-  m_oldx = x;
-  m_oldy = y;
-  m_copy = event.ControlDown();
-  if (event.LeftDown() && m_tt == ttOut) {
-	// select object
-	DoSelect(HitTest(x, y), true);
-    // change mouse cursor
-	event.m_leftDown = false;
-	OnMouseMove(event);
-  }
-}
-
-void MenuEditor::OnMouseMove(wxMouseEvent &event)
-{
-  if (!m_menu)
-	return;
-  
-  int x = event.GetX();
-  int y = event.GetY();
-  
-  if (!event.LeftIsDown())
-  {
-    m_tt = GetTransformType(x, y);
-    // change mouse cursor
-    SetMouseCursor(m_tt);
-	// object pointed
-	wxString pointed = HitTest(x, y);
-	if (GetPointed() != pointed)
-	{
-	  m_pointed = pointed;
-	  wxCommandEvent evt(EVT_COMMAND_MENUEDITOR_OBJECT_POINTED, this->GetId());
-	  GetEventHandler()->ProcessEvent(evt);
+	Menu menu;
+	MenuObject obj(&menu, fname);
+	MenuObjectParam* initParam = obj.GetInitParam();
+	if (initParam) {
+		if (initParam->type == wxT("text") || initParam->type == wxT("string"))
+			param = wxGetTextFromUser(_("Please type in text to insert"), _("Input text"), _T("Text"), this);
+		else if (initParam->type == wxT("image"))
+			param = wxFileSelector( _ ("Please choose an image to insert") , wxT("") ,wxGetCwd (), wxT(""),
+					_("Image Files ") + wxImage::GetImageExtWildcard(), wxOPEN|wxFILE_MUST_EXIST);
+		else
+			param = _("Text");
+		if (!param.length())
+			return false;
 	}
-    return;
-  }
-  
-  // left mouse button is down => move or resize
-  if (m_tt == ttIn)
-  {
-	// move Object
-    int x1 = x - m_oldx;
-    int y1 = y - m_oldy;
-    if (MoveObject(x1, y1))
-    {
-      DoSelect(GetSelected(), true); // refresh
-      m_eventTimer.Start(500, wxTIMER_ONE_SHOT); // send changed event
-    }
-    m_oldx += x1;
-    m_oldy += y1;
-  }
-  else if (m_tt != ttOut) // m_tt=TL,TR,BL,BR
-  {
-    // resize object
-    if (ResizeObject(x, y, m_tt))
-	{
-      DoSelect(GetSelected(), true); // refresh
-      m_eventTimer.Start(500, wxTIMER_ONE_SHOT); // send changed event
-    }
-  }
-}
 
-void MenuEditor::OnMouseRightButton(wxMouseEvent &event)
-{
-  SetFocus();
-  if (event.RightDown() || !m_menu)
-	return;
-  // select Object
-  wxString sel_old = GetSelected();
-  DoSelect(HitTest(event.GetX(),event.GetY()), true);
-  m_menuPos = event.GetPosition();
-  if (event.RightUp())
-	PopupMenu(GetSelected().length() ? m_objMenu : m_pmenu, event.GetPosition());
-}
-
-TransformType MenuEditor::GetTransformType(int x, int y)
-{
-  if (!m_menu || !GetSelected().length())
-	return ttOut;
-  
-  // test all covered object
-  if (HitTest(x,y) != GetSelected())
-    return ttOut; // outside of the selected object
-  
-  MenuObject* obj = m_menu->GetObject(GetSelected());
-  if (!obj)
-    return ttOut;
-  wxRect rect = ScaleRect(obj->GetBBox(), GetScale());
-  wxRect rect2 = rect;
-  rect2.Deflate(3,3);
-  if (!rect2.Contains(x,y))
-  {
-    int x1 = rect.x + rect.width/2;
-    int y1 = rect.y + rect.height/2;
-    if ((x < x1) && (y < y1)) 
-      return ttTL; // top-left
-    else if ((x > x1) && (y < y1)) 
-      return ttTR; // top-right
-    else if ((x < x1) && (y > y1)) 
-      return ttBL; // bottom-left
-    else if ((x > x1) && (y > y1)) 
-      return ttBR; // bottom-right
-  }
-  
-  return ttIn; // inside of the selected object
-}
-
-void MenuEditor::SetMouseCursor(TransformType tt)
-{
-  switch (tt)
-  {
-    case ttOut: SetCursor(wxCursor(wxCURSOR_ARROW)); break;
-#ifdef __WXMSW__
-    case ttIn: SetCursor(wxCursor(_T("wxCURSOR_MOVE"),wxBITMAP_TYPE_CUR_RESOURCE)); break;
-    case ttTL:
-    case ttBR: SetCursor(wxCursor(wxCURSOR_SIZENWSE)); break;
-    case ttTR:
-    case ttBL: SetCursor(wxCursor(wxCURSOR_SIZENESW)); break;
-#else
-    case ttIn: SetCursor(wxImage(CURSOR_FILE(_T("move.cur"))));break;
-    case ttTL:
-    case ttBR: SetCursor(wxImage(CURSOR_FILE(_T("nwse.cur"))));break;
-    case ttTR:
-    case ttBL: SetCursor(wxImage(CURSOR_FILE(_T("nesw.cur"))));break;
-#endif
-  }
-}
-
-bool MenuEditor::MoveObject(int& x, int& y)
-{
-  x = (int) round(x/GetScale());
-  y = (int) round(y/GetScale());
-  bool res = MoveObjectInt(x,y);
-  x = (int) round(x*GetScale());
-  y = (int) round(y*GetScale());
-  return res;
-}
-
-bool MenuEditor::MoveObjectInt(int& x, int& y)
-{
-  if (!m_menu)
-	return false;
-  
-  MenuObject* obj = m_menu->GetObject(GetSelected());
-  if (!obj)
-    return false;
-  wxRect rect = obj->GetBBox();
-  
-  if (m_copy) {
-	  wxString id = m_menu->AddObject(obj->GetXML(DVDSTYLER_XML, true), true);
-	  if (id.length() == 0)
-		  return false;
-	  obj = m_menu->GetObject(id);
-	  if (obj) {
-	  	  obj->SetX(rect.x);
-	  	  obj->SetY(rect.y);
-	  }
-	  DoSelect(id, true);
-	  m_pointed = id;
-	  wxCommandEvent evt(EVT_COMMAND_MENUEDITOR_OBJECT_POINTED, this->GetId());
-	  GetEventHandler()->ProcessEvent(evt);
-	  m_copy = false;
-  }
-  
-  rect.x += x;
-  rect.y += y;
-  
-  if (m_grid>0)
-  {
-	rect.y = rect.y>>m_grid<<m_grid;
-	rect.x = rect.x>>m_grid<<m_grid;
-  }
-  
-  if (obj->IsButton())
-  {
-  	if (rect.x<0)
-  	  rect.x = 0;
-  	else if (rect.x + rect.width >= (int)m_doc->GetRootElement()->GetWidth().GetBaseVal())
-  	  rect.x = (int)m_doc->GetRootElement()->GetWidth().GetBaseVal() - rect.width - 1;
-  	if (rect.y<0)
-  	  rect.y = 0;
-  	else if (rect.y + rect.height >= (int)m_doc->GetRootElement()->GetHeight().GetBaseVal())
-  	  rect.y = (int)m_doc->GetRootElement()->GetHeight().GetBaseVal() - rect.height - 1;
-  }
-  else
-  {
-	if (rect.x + rect.width < 8)
-      rect.x = 8 - rect.width;
-	else if (rect.x > (int)m_doc->GetRootElement()->GetWidth().GetBaseVal()-8)
-	  rect.x = (int)m_doc->GetRootElement()->GetWidth().GetBaseVal()-8;
-	if (rect.y + rect.height < 8)
-	  rect.y = 8 - rect.height;
-	else if (rect.y > (int)m_doc->GetRootElement()->GetHeight().GetBaseVal()-8)
-	  rect.y = (int)m_doc->GetRootElement()->GetHeight().GetBaseVal()-8;
-  }
-  
-  x = rect.x - obj->GetX();
-  y = rect.y - obj->GetY();
-  
-  if (obj->GetX() != rect.x || obj->GetY() != rect.y)
-  {
-	obj->SetX(rect.x);
-    obj->SetY(rect.y);
+	DoSelect(m_menu->AddObject(fname, param, x, y), true, true);
 	return true;
-  }
-  
-  return false;
+}
+
+wxString MenuEditor::HitTest(int x, int y) {
+	x = (int) (x / GetScale());
+	y = (int) (y / GetScale());
+	wxString id;
+	// at frist try to find a button
+	bool button = true;
+	wxSVGUseElement* child = (wxSVGUseElement*) m_doc->GetElementById(wxT("buttons"))->GetChildren();
+	while (child || button && id.length() == 0) {
+		if (!child) {
+			// no button was found, try to find a object
+			child = (wxSVGUseElement*) m_doc->GetElementById(wxT("objects"))->GetChildren();
+			button = false;
+			continue;
+		}
+		if (child->GetType() == wxSVGXML_ELEMENT_NODE && child->GetDtd() == wxSVG_USE_ELEMENT) {
+			wxSVGRect bbox = child->GetBBox();
+			if (!bbox.IsEmpty()
+					&& x >= bbox.GetX() && x <= bbox.GetX() + bbox.GetWidth()
+					&& y >= bbox.GetY() && y <= bbox.GetY() + bbox.GetHeight())
+				id = child->GetId();
+		}
+		child = (wxSVGUseElement*) child->GetNext();
+	}
+	return id;
+}
+
+void MenuEditor::DoSelect(wxString id, bool refresh, bool sendEvent) {
+	if (!m_menu)
+		return;
+
+	m_selected = id;
+
+	// remove old selection
+	wxSVGRectElement* selRect = (wxSVGRectElement*) m_doc->GetElementById(wxT("selection"));
+	if (selRect) {
+		if (refresh)
+			RefreshRect(ScaleRect(selRect->GetBBox(), GetScale()).Inflate(4));
+		selRect->GetParent()->RemoveChild(selRect);
+	}
+
+	// create selection rectangle
+	if (id.length()) {
+		wxSVGUseElement* elem = (wxSVGUseElement*) m_doc->GetElementById(id);
+		if (!elem || elem->GetDtd() != wxSVG_USE_ELEMENT)
+			return;
+		wxSVGRect bbox = elem->GetBBox();
+
+		selRect = new wxSVGRectElement;
+		selRect->SetId(wxT("selection"));
+		selRect->SetX(bbox.GetX());
+		selRect->SetY(bbox.GetY());
+		selRect->SetWidth(bbox.GetWidth());
+		selRect->SetHeight(bbox.GetHeight());
+		selRect->SetStroke(wxSVGPaint(255, 0, 0));
+		selRect->SetFill(wxSVGPaint());
+		m_doc->GetRootElement()->AppendChild(selRect);
+
+		if (refresh)
+			RefreshRect(ScaleRect(selRect->GetBBox(), GetScale()).Inflate(4));
+	}
+
+	if (sendEvent)
+		SendChangedEvent();
+}
+
+void MenuEditor::OnMouseLeftButton(wxMouseEvent &event) {
+	SetFocus();
+	if (!m_menu)
+		return;
+	int x = event.GetX();
+	int y = event.GetY();
+	m_tt = GetTransformType(x, y);
+	m_oldx = x;
+	m_oldy = y;
+	m_copy = event.ControlDown();
+	if (event.LeftDown() && m_tt == ttOut) {
+		// select object
+		DoSelect(HitTest(x, y), true);
+		// change mouse cursor
+		event.m_leftDown = false;
+		OnMouseMove(event);
+	}
+}
+
+void MenuEditor::OnMouseMove(wxMouseEvent &event) {
+	if (!m_menu)
+		return;
+
+	int x = event.GetX();
+	int y = event.GetY();
+
+	if (!event.LeftIsDown()) {
+		m_tt = GetTransformType(x, y);
+		// change mouse cursor
+		SetMouseCursor(m_tt);
+		// object pointed
+		wxString pointed = HitTest(x, y);
+		if (GetPointed() != pointed) {
+			m_pointed = pointed;
+			wxCommandEvent evt(EVT_COMMAND_MENUEDITOR_OBJECT_POINTED, this->GetId());
+			GetEventHandler()->ProcessEvent(evt);
+		}
+		return;
+	}
+
+	// left mouse button is down => move or resize
+	if (m_tt == ttIn) {
+		// move Object
+		int x1 = x - m_oldx;
+		int y1 = y - m_oldy;
+		if (MoveObject(x1, y1)) {
+			DoSelect(GetSelected(), true); // refresh
+			m_eventTimer.Start(500, wxTIMER_ONE_SHOT); // send changed event
+		}
+		m_oldx += x1;
+		m_oldy += y1;
+	} else if (m_tt != ttOut) { // m_tt=TL,TR,BL,BR
+		// resize object
+		if (ResizeObject(x, y, m_tt)) {
+			DoSelect(GetSelected(), true); // refresh
+			m_eventTimer.Start(500, wxTIMER_ONE_SHOT); // send changed event
+		}
+	}
+}
+
+void MenuEditor::OnMouseRightButton(wxMouseEvent &event) {
+	SetFocus();
+	if (event.RightDown() || !m_menu)
+		return;
+	// select Object
+	wxString sel_old = GetSelected();
+	DoSelect(HitTest(event.GetX(), event.GetY()), true);
+	m_menuPos = event.GetPosition();
+	if (event.RightUp())
+		PopupMenu(GetSelected().length() ? m_objMenu : m_pmenu, event.GetPosition());
+}
+
+TransformType MenuEditor::GetTransformType(int x, int y) {
+	if (!m_menu || !GetSelected().length())
+		return ttOut;
+
+	// test all covered object
+	if (HitTest(x, y) != GetSelected())
+		return ttOut; // outside of the selected object
+
+	MenuObject* obj = m_menu->GetObject(GetSelected());
+	if (!obj)
+		return ttOut;
+	wxRect rect = ScaleRect(obj->GetBBox(), GetScale());
+	wxRect rect2 = rect;
+	rect2.Deflate(3, 3);
+	if (!rect2.Contains(x, y)) {
+		int x1 = rect.x + rect.width / 2;
+		int y1 = rect.y + rect.height / 2;
+		if ((x < x1) && (y < y1))
+			return ttTL; // top-left
+		else if ((x > x1) && (y < y1))
+			return ttTR; // top-right
+		else if ((x < x1) && (y > y1))
+			return ttBL; // bottom-left
+		else if ((x > x1) && (y > y1))
+			return ttBR; // bottom-right
+	}
+
+	return ttIn; // inside of the selected object
+}
+
+void MenuEditor::SetMouseCursor(TransformType tt) {
+	switch (tt) {
+		case ttOut: SetCursor(wxCursor(wxCURSOR_ARROW)); break;
+#ifdef __WXMSW__
+		case ttIn: SetCursor(wxCursor(_T("wxCURSOR_MOVE"),wxBITMAP_TYPE_CUR_RESOURCE)); break;
+		case ttTL:
+		case ttBR: SetCursor(wxCursor(wxCURSOR_SIZENWSE)); break;
+		case ttTR:
+		case ttBL: SetCursor(wxCursor(wxCURSOR_SIZENESW)); break;
+#else
+		case ttIn: SetCursor(wxImage(CURSOR_FILE(_T("move.cur"))));break;
+		case ttTL:
+		case ttBR: SetCursor(wxImage(CURSOR_FILE(_T("nwse.cur"))));break;
+		case ttTR:
+		case ttBL: SetCursor(wxImage(CURSOR_FILE(_T("nesw.cur"))));break;
+#endif
+	}
+}
+
+bool MenuEditor::MoveObject(int& x, int& y) {
+	x = (int) round(x / GetScale());
+	y = (int) round(y / GetScale());
+	bool res = MoveObjectInt(x, y);
+	x = (int) round(x * GetScale());
+	y = (int) round(y * GetScale());
+	return res;
+}
+
+bool MenuEditor::MoveObjectInt(int& x, int& y) {
+	if (!m_menu)
+		return false;
+
+	MenuObject* obj = m_menu->GetObject(GetSelected());
+	if (!obj)
+		return false;
+	wxRect rect = obj->GetBBox();
+
+	if (m_copy) {
+		wxString id = m_menu->AddObject(obj->GetXML(DVDSTYLER_XML, true), true);
+		if (id.length() == 0)
+			return false;
+		obj = m_menu->GetObject(id);
+		if (obj) {
+			obj->SetX(rect.x);
+			obj->SetY(rect.y);
+		}
+		DoSelect(id, true);
+		m_pointed = id;
+		wxCommandEvent
+				evt(EVT_COMMAND_MENUEDITOR_OBJECT_POINTED, this->GetId());
+		GetEventHandler()->ProcessEvent(evt);
+		m_copy = false;
+	}
+
+	rect.x += x;
+	rect.y += y;
+
+	if (m_grid > 0) {
+		rect.y = rect.y >> m_grid << m_grid;
+		rect.x = rect.x >> m_grid << m_grid;
+	}
+
+	if (obj->IsButton()) {
+		if (rect.x < 0)
+			rect.x = 0;
+		else if (rect.x + rect.width >= (int) m_doc->GetRootElement()->GetWidth().GetBaseVal())
+			rect.x = (int) m_doc->GetRootElement()->GetWidth().GetBaseVal() - rect.width - 1;
+		if (rect.y < 0)
+			rect.y = 0;
+		else if (rect.y + rect.height >= (int) m_doc->GetRootElement()->GetHeight().GetBaseVal())
+			rect.y = (int) m_doc->GetRootElement()->GetHeight().GetBaseVal() - rect.height - 1;
+	} else {
+		if (rect.x + rect.width < 8)
+			rect.x = 8 - rect.width;
+		else if (rect.x > (int) m_doc->GetRootElement()->GetWidth().GetBaseVal() - 8)
+			rect.x = (int) m_doc->GetRootElement()->GetWidth().GetBaseVal() - 8;
+		if (rect.y + rect.height < 8)
+			rect.y = 8 - rect.height;
+		else if (rect.y > (int) m_doc->GetRootElement()->GetHeight().GetBaseVal() - 8)
+			rect.y = (int) m_doc->GetRootElement()->GetHeight().GetBaseVal() - 8;
+	}
+
+	x = rect.x - obj->GetX();
+	y = rect.y - obj->GetY();
+
+	if (obj->GetX() != rect.x || obj->GetY() != rect.y) {
+		obj->SetX(rect.x);
+		obj->SetY(rect.y);
+		return true;
+	}
+
+	return false;
 }
 
 bool MenuEditor::ResizeObject(int x, int y, TransformType transformType)
@@ -720,64 +684,62 @@ bool MenuEditor::ResizeObject(int x, int y, TransformType transformType)
   return ResizeObjectInt(x, y, transformType);
 }
 
-bool MenuEditor::ResizeObjectInt(int x, int y, TransformType transformType)
-{
-  if (!m_menu)
+bool MenuEditor::ResizeObjectInt(int x, int y, TransformType transformType) {
+	if (!m_menu)
+		return false;
+
+	MenuObject* obj = m_menu->GetObject(GetSelected());
+	if (!obj)
+		return false;
+	wxRect rect = obj->GetBBox();
+
+	if ((transformType == ttBR || transformType == ttTR) && x < 8)
+		x = 8;
+	if ((transformType == ttBL || transformType == ttTL)
+			&& x > (int) m_doc->GetRootElement()->GetWidth().GetBaseVal() - 8)
+		x = (int) m_doc->GetRootElement()->GetWidth().GetBaseVal() - 8;
+	if ((transformType == ttBL || transformType == ttBR) && y < 8)
+		y = 8;
+	if ((transformType == ttTL || transformType == ttTR)
+			&& y > (int) m_doc->GetRootElement()->GetHeight().GetBaseVal() - 8)
+		y = (int) m_doc->GetRootElement()->GetHeight().GetBaseVal() - 8;
+
+	int sx, sy;
+
+	if ((transformType == ttBR) || (transformType == ttTR))
+		sx = x - rect.x + 1;
+	else
+		sx = rect.x + rect.width - x + 1;
+
+	if ((transformType == ttBR) || (transformType == ttBL))
+		sy = y - rect.y + 1;
+	else
+		sy = rect.y + rect.height - y + 1;
+
+	if (sx <= 4)
+		sx = 4;
+	if (sy <= 4)
+		sy = 4;
+
+	obj->SetDefaultSize(false);
+	obj->FixSize(sx, sy);
+
+	// set new x, y, width, height
+	if ((transformType == ttTL) || (transformType == ttBL))
+		rect.x += rect.width - sx;
+	if ((transformType == ttTL) || (transformType == ttTR))
+		rect.y += rect.height - sy;
+	rect.width = sx;
+	rect.height = sy;
+
+	if (obj->GetBBox() != rect) {
+		obj->SetX(rect.x);
+		obj->SetY(rect.y);
+		obj->SetWidth(rect.width);
+		obj->SetHeight(rect.height);
+		return true;
+	}
 	return false;
-  
-  MenuObject* obj = m_menu->GetObject(GetSelected());
-  if (!obj)
-    return false;
-  wxRect rect = obj->GetBBox();
-  
-  if ((transformType == ttBR || transformType == ttTR) && x<8)
-    x = 8;
-  if ((transformType == ttBL || transformType == ttTL) &&
-      x > (int)m_doc->GetRootElement()->GetWidth().GetBaseVal() - 8)
-    x = (int)m_doc->GetRootElement()->GetWidth().GetBaseVal() - 8;
-  if ((transformType == ttBL || transformType == ttBR) && y<8)
-    y = 8;
-  if ((transformType == ttTL || transformType == ttTR) &&
-      y > (int)m_doc->GetRootElement()->GetHeight().GetBaseVal() - 8)
-    y = (int)m_doc->GetRootElement()->GetHeight().GetBaseVal() - 8;
-  
-  int sx, sy;
-  
-  if ((transformType == ttBR) || (transformType == ttTR))
-    sx = x - rect.x + 1;
-  else
-    sx = rect.x + rect.width - x + 1;
-
-  if ((transformType == ttBR) || (transformType == ttBL))
-    sy = y - rect.y + 1;
-  else
-    sy = rect.y + rect.height - y + 1;
-  
-  if (sx <= 4)
-    sx = 4;
-  if (sy <= 4)
-    sy = 4;
-  
-  obj->SetDefaultSize(false);
-  obj->FixSize(sx, sy);
-
-  // set new x, y, width, height
-  if ((transformType == ttTL) || (transformType == ttBL))
-    rect.x += rect.width - sx;
-  if ((transformType == ttTL) || (transformType == ttTR))
-    rect.y += rect.height - sy;
-  rect.width = sx;
-  rect.height = sy;
-  
-  if (obj->GetBBox() != rect)
-  {
-    obj->SetX(rect.x);
-    obj->SetY(rect.y);
-    obj->SetWidth(rect.width);
-    obj->SetHeight(rect.height);
-	return true;
-  }
-  return false;
 }
 
 //////////////////////////////////////////////////////////////////////////////
@@ -858,51 +820,41 @@ void MenuEditor::ShowGrid() {
 
 //////////////////////////////////////////////////////////////////////////////
 
-void MenuEditor::OnEventTimer(wxTimerEvent& event)
-{
-  SendChangedEvent();
+void MenuEditor::OnEventTimer(wxTimerEvent& event) {
+	SendChangedEvent();
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-void MenuEditor::OnMouseDClick(wxMouseEvent& WXUNUSED(event))
-{
-  wxCommandEvent evt;
-  OnProperties(evt);
+void MenuEditor::OnMouseDClick(wxMouseEvent& WXUNUSED(event)) {
+	wxCommandEvent evt;
+	OnProperties(evt);
 }
 
-void MenuEditor::OnKeyDown(wxKeyEvent& event)
-{
-  if (event.ControlDown() && event.GetKeyCode() == 'V')
-  {
-    m_menuPos = wxPoint(-1,-1);
-	wxCommandEvent evt;
-	OnObjectPaste(evt);
-	return;
-  }
-  if (GetSelected().length())
-  {
-	if (event.GetKeyCode() == WXK_DELETE)
-	{
-	  wxCommandEvent evt;
-	  OnObjectDelete(evt);
-	  return;
+void MenuEditor::OnKeyDown(wxKeyEvent& event) {
+	if (event.ControlDown() && event.GetKeyCode() == 'V') {
+		m_menuPos = wxPoint(-1, -1);
+		wxCommandEvent evt;
+		OnObjectPaste(evt);
+		return;
 	}
-	else if (event.ControlDown() && event.GetKeyCode() == 'C')
-	{
-	  wxCommandEvent evt;
-	  OnObjectCopy(evt);
-	  return;
+	if (GetSelected().length()) {
+		if (event.GetKeyCode() == WXK_DELETE) {
+			wxCommandEvent evt;
+			OnObjectDelete(evt);
+			return;
+		} else if (event.ControlDown() && event.GetKeyCode() == 'C') {
+			wxCommandEvent evt;
+			OnObjectCopy(evt);
+			return;
+		} else if (event.ControlDown() && event.GetKeyCode() == 'X') {
+			wxCommandEvent evt;
+			OnObjectCopy(evt);
+			OnObjectDelete(evt);
+			return;
+		}
 	}
-    else if (event.ControlDown() && event.GetKeyCode() == 'X')
-	{
-	  wxCommandEvent evt;
-	  OnObjectCopy(evt);
-      OnObjectDelete(evt);
-	  return;
-	}
-  }
-  event.Skip();
+	event.Skip();
 }
 
 //////////////////////////////////////////////////////////////////////////////
@@ -939,119 +891,100 @@ void MenuEditor::OnObjectPaste(wxCommandEvent& WXUNUSED(event)) {
 	Update();
 }
 
-void MenuEditor::OnUpdateUIObjectPaste(wxUpdateUIEvent& event)
-{
-  event.Enable(m_menu &&
-    (wxTheClipboard->IsSupported(wxDF_TEXT) ||
-	 wxTheClipboard->IsSupported(wxDataFormat(DATAFORMAT_MENU_OBJECT))));
+void MenuEditor::OnUpdateUIObjectPaste(wxUpdateUIEvent& event) {
+	event.Enable(m_menu && (wxTheClipboard->IsSupported(wxDF_TEXT)
+			|| wxTheClipboard->IsSupported(wxDataFormat(DATAFORMAT_MENU_OBJECT))));
 }
 
-void MenuEditor::OnObjectCut(wxCommandEvent& WXUNUSED(event))
-{
-  wxCommandEvent evt;
-  OnObjectCopy(evt);
-  OnObjectDelete(evt);
+void MenuEditor::OnObjectCut(wxCommandEvent& WXUNUSED(event)) {
+	wxCommandEvent evt;
+	OnObjectCopy(evt);
+	OnObjectDelete(evt);
 }
 
-void MenuEditor::OnObjectDelete(wxCommandEvent& WXUNUSED(event))
-{
-  m_menu->RemoveObject(GetSelected());
-  DoSelect(wxT(""), true, true);
+void MenuEditor::OnObjectDelete(wxCommandEvent& WXUNUSED(event)) {
+	m_menu->RemoveObject(GetSelected());
+	DoSelect(wxT(""), true, true);
 }
 
-void MenuEditor::OnObjectToFront(wxCommandEvent& WXUNUSED(event))
-{
-  m_menu->ObjectToFront(GetSelected());
-  DoSelect(GetSelected(), true, true);
+void MenuEditor::OnObjectToFront(wxCommandEvent& WXUNUSED(event)) {
+	m_menu->ObjectToFront(GetSelected());
+	DoSelect(GetSelected(), true, true);
 }
 
-void MenuEditor::OnObjectForward(wxCommandEvent& WXUNUSED(event))
-{
-  m_menu->ObjectForward(GetSelected());
-  DoSelect(GetSelected(), true, true);
+void MenuEditor::OnObjectForward(wxCommandEvent& WXUNUSED(event)) {
+	m_menu->ObjectForward(GetSelected());
+	DoSelect(GetSelected(), true, true);
 }
 
-void MenuEditor::OnObjectBackward(wxCommandEvent& WXUNUSED(event))
-{
-  m_menu->ObjectBackward(GetSelected());
-  DoSelect(GetSelected(), true, true);
+void MenuEditor::OnObjectBackward(wxCommandEvent& WXUNUSED(event)) {
+	m_menu->ObjectBackward(GetSelected());
+	DoSelect(GetSelected(), true, true);
 }
 
-void MenuEditor::OnObjectToBack(wxCommandEvent& WXUNUSED(event))
-{
-  m_menu->ObjectToBack(GetSelected());
-  DoSelect(GetSelected(), true, true);
+void MenuEditor::OnObjectToBack(wxCommandEvent& WXUNUSED(event)) {
+	m_menu->ObjectToBack(GetSelected());
+	DoSelect(GetSelected(), true, true);
 }
 
-void MenuEditor::OnUpdateUIObjectForward(wxUpdateUIEvent& event)
-{
-  event.Enable(!m_menu->IsLastObject(GetSelected()));
+void MenuEditor::OnUpdateUIObjectForward(wxUpdateUIEvent& event) {
+	event.Enable(!m_menu->IsLastObject(GetSelected()));
 }
 
-void MenuEditor::OnUpdateUIObjectBackward(wxUpdateUIEvent& event)
-{
-  event.Enable(!m_menu->IsFirstObject(GetSelected()));
+void MenuEditor::OnUpdateUIObjectBackward(wxUpdateUIEvent& event) {
+	event.Enable(!m_menu->IsFirstObject(GetSelected()));
 }
 
-void MenuEditor::OnProperties(wxCommandEvent& WXUNUSED(event))
-{
-  if (!m_menu)
-	return;
-  if (!GetSelected().length()) // menu properties
-  {
-	MenuPropDlg propDlg(this, m_dvd, m_tsi, m_pgci);
-	if (propDlg.ShowModal() == wxID_OK)
-	  Update();
-  }
-  else // object properties
-  {
-    MenuObject* obj = m_menu->GetObject(GetSelected());
-    if (!obj)
-      return;
-    MenuObjectPropDlg propDlg(this, GetSelected(),
-      m_menu, m_dvd, m_tsi, m_pgci);
-    if (propDlg.ShowModal() == wxID_OK)
-      DoSelect(GetSelected(), true, true);
-  }
+void MenuEditor::OnProperties(wxCommandEvent& WXUNUSED(event)) {
+	if (!m_menu)
+		return;
+	if (!GetSelected().length()) {
+		// show menu properties
+		MenuPropDlg propDlg(this, m_dvd, m_tsi, m_pgci);
+		if (propDlg.ShowModal() == wxID_OK)
+			Update();
+	} else {
+		// show object properties
+		MenuObject* obj = m_menu->GetObject(GetSelected());
+		if (!obj)
+			return;
+		MenuObjectPropDlg propDlg(this, GetSelected(), m_menu, m_dvd, m_tsi, m_pgci);
+		if (propDlg.ShowModal() == wxID_OK)
+			DoSelect(GetSelected(), true, true);
+	}
 }
 
-void MenuEditor::OnSafeTV(wxCommandEvent& event)
-{
+void MenuEditor::OnSafeTV(wxCommandEvent& event) {
 	m_safeTV = !m_safeTV;
 	ShowSafeTV();
 }
 
-void MenuEditor::OnShowGrid(wxCommandEvent& event)
-{
+void MenuEditor::OnShowGrid(wxCommandEvent& event) {
 	m_showGrid = !m_showGrid;
 	ShowGrid();
 }
 
-void MenuEditor::OnShowGridOverObjects(wxCommandEvent& event)
-{
+void MenuEditor::OnShowGridOverObjects(wxCommandEvent& event) {
 	m_gridOverObejcts = !m_gridOverObejcts;
 	ShowGrid();
 }
 
-void MenuEditor::OnGridMenu(wxCommandEvent& event)
-{
-  long id = event.GetId() - MENUEDITOR_GRIDMENU_ITEM_ID;
-  m_grid = id;
-  if (m_grid>0)
-    m_grid += 1;
-  if (m_showGrid)
-  	ShowGrid();
+void MenuEditor::OnGridMenu(wxCommandEvent& event) {
+	long id = event.GetId() - MENUEDITOR_GRIDMENU_ITEM_ID;
+	m_grid = id;
+	if (m_grid > 0)
+		m_grid += 1;
+	if (m_showGrid)
+		ShowGrid();
 }
 
-void MenuEditor::OnAddObject(wxCommandEvent& event)
-{
-  long n = event.GetId() - MENUEDITOR_ADDMENU_OBJECT_ID;
-  AddObject(m_addMenuObjects[n],
-    (int)(m_menuPos.x/GetScale()), (int)(m_menuPos.y/GetScale()));
+void MenuEditor::OnAddObject(wxCommandEvent& event) {
+	long n = event.GetId() - MENUEDITOR_ADDMENU_OBJECT_ID;
+	AddObject(m_addMenuObjects[n], (int) (m_menuPos.x / GetScale()),
+			(int) (m_menuPos.y / GetScale()));
 }
 
-void MenuEditor::SendChangedEvent()
-{
-  wxCommandEvent evt(EVT_COMMAND_MENUEDITOR_MENU_CHANGED, this->GetId());
-  GetEventHandler()->ProcessEvent(evt);
+void MenuEditor::SendChangedEvent() {
+	wxCommandEvent evt(EVT_COMMAND_MENUEDITOR_MENU_CHANGED, this->GetId());
+	GetEventHandler()->ProcessEvent(evt);
 }
