@@ -3,7 +3,7 @@
 // Purpose:     DVD menu button properties dialog
 // Author:      Alex Thuering
 // Created:	20.11.2003
-// RCS-ID:      $Id: MenuObjectPropDlg.cpp,v 1.23 2009/03/04 20:38:36 ntalex Exp $
+// RCS-ID:      $Id: MenuObjectPropDlg.cpp,v 1.25 2009/03/29 19:34:45 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -33,13 +33,15 @@ BEGIN_EVENT_TABLE(MenuObjectPropDlg, wxPropDlg)
 	EVT_CHOICE(PGCI_CHOICE_ID, MenuObjectPropDlg::OnChangeTarget)
 END_EVENT_TABLE()
 
-MenuObjectPropDlg::MenuObjectPropDlg(wxWindow* parent,
-		wxString id, Menu* menu, DVD* dvd, int tsi, int pgci): wxPropDlg(parent) {
+MenuObjectPropDlg::MenuObjectPropDlg(wxWindow* parent, wxString id, Menu* menu, DVD* dvd, int tsi, int pgci):
+		wxPropDlg(parent, _("Properties") + wxString(wxT(" - ")) + menu->GetObject(id)->GetId(true)) {
 	m_id = id;
 	m_menu = menu;
 	m_dvd = dvd;
 	m_tsi = tsi;
 	m_pgci = pgci;
+	m_imageEdit = NULL;
+	m_imageRadio = NULL;
 	m_videoFrameRadio = NULL;
 	m_object = menu->GetObject(id);
 	Create();
@@ -77,8 +79,7 @@ void MenuObjectPropDlg::CreatePropPanel(wxSizer* sizer) {
 					|| GetChapter() != m_object->GetActionChapter())
 				customAction = m_object->GetAction();
 		}
-		AddRadioProp(grid, _("Custom"), customAction.length(), 0,
-				false, CUSTOM_ACTION_RADIO_ID);
+		AddRadioProp(grid, _("Custom"), customAction.length()>0, 0, false, CUSTOM_ACTION_RADIO_ID);
 		AddTextProp(grid, wxT(""), customAction);
 		m_customActionEdit = (wxTextCtrl*) m_controls[m_controls.Count()-1];
 		mainSizer->Add(grid, 0, wxEXPAND|wxBOTTOM, 8);
@@ -162,11 +163,13 @@ void MenuObjectPropDlg::CreatePropPanel(wxSizer* sizer) {
 					wxString fname = imageFile.Find(wxT('#')) != -1 ? imageFile.BeforeLast(wxT('#')) : imageFile;
 					bool image = !hasVideo || fname != GetVideoFilename();
 					AddRadioProp(grid, wxString(_("Image")) + wxT(":"), image, wxRB_GROUP, false, IMAGE_RADIO_ID);
+					m_imageRadio = (wxRadioButton*) GetLastControl();
 					AddFileProp(grid, wxT(""), image ? imageFile : wxT(""), wxFD_OPEN, wxT("..."),
 							_("Image Files ") + wxImage::GetImageExtWildcard()
 							+ wxT("|") + wxString(_("All Files")) + wxT(" (*.*)|*.*"));
+					m_imageEdit = (wxTextCtrl*) GetLastControl();
 					AddRadioProp(grid, wxString(_("Video Frame")) + wxT(":"), !image, 0, false, VIDEOFRAME_RADIO_ID);
-					m_videoFrameRadio = (wxRadioButton*) m_controls[m_controls.Count()-1];
+					m_videoFrameRadio = (wxRadioButton*) GetLastControl();
 					if (!hasVideo)
 						m_videoFrameRadio->Enable(false);
 					bool vfAuto = imageFile == GetVideoFilename(true) || imageFile.length() == 0;
@@ -330,6 +333,11 @@ void MenuObjectPropDlg::OnJumpActionSelected(wxCommandEvent& WXUNUSED(evt)) {
 	m_targetChoice->Enable(true);
 	m_chapterChoice->Enable(m_chapterChoice->GetCount() > 1);
 	m_customActionEdit->Enable(false);
+	if (m_videoFrameRadio) {
+		m_videoFrameRadio->Enable(GetVideoFilename().length());
+		if (GetVideoFilename().length() && m_imageEdit->GetValue().length() == 0)
+			m_videoFrameRadio->SetValue(true);
+	}
 }
 
 void MenuObjectPropDlg::OnCustomActionSelected(wxCommandEvent& WXUNUSED(evt)) {
@@ -343,6 +351,10 @@ void MenuObjectPropDlg::OnCustomActionSelected(wxCommandEvent& WXUNUSED(evt)) {
 		m_customActionEdit->SetValue(m_object->GetAction());
 		m_object->SetActionTsi(tsi);
 	}
+	if (m_videoFrameRadio) {
+		m_videoFrameRadio->Enable(false);
+		m_imageRadio->SetValue(true);
+	}
 }
 
 wxString MenuObjectPropDlg::GetVideoFilename(bool withTimestamp, long position) {
@@ -368,16 +380,6 @@ void MenuObjectPropDlg::OnCustomSizeSelected(wxCommandEvent& WXUNUSED(evt)) {
 	m_heightEdit->Enable(true);
 }
 
-
-void MenuObjectPropDlg::AddStaticLine(wxSizer* sizer, wxString caption) {
-	wxBoxSizer* sizer2 = new wxBoxSizer(wxHORIZONTAL);
-	wxStaticText* staticText = new wxStaticText(this, -1, caption);
-	staticText->SetFont(wxFont(12, wxDEFAULT, wxNORMAL, wxBOLD, 0, _T("")));
-	sizer2->Add(staticText, 0, wxRIGHT | wxTOP | wxBOTTOM, 5);
-	sizer2->Add(new wxStaticLine(this, -1), 1, wxALIGN_CENTER_VERTICAL, 5);
-	sizer->Add(sizer2, 0, wxEXPAND, 0);
-}
-
 void MenuObjectPropDlg::AddDirectionProp(wxSizer* sizer, ButtonDirCommand dir) {
 	wxFlexGridSizer* grid = new wxFlexGridSizer(2, 1, 4, 16);
 	AddChoiceProp(grid, wxT(""), wxT(""), wxArrayString());
