@@ -1,15 +1,15 @@
 /////////////////////////////////////////////////////////////////////////////
-// Name:        NewPropjectDlg.h
+// Name:        NewProjectDlg.h
 // Purpose:     New project dialog
 // Author:      Alex Thuering
-// Created:	29.10.2006
-// RCS-ID:      $Id: NewProjectDlg.h,v 1.5 2008/12/26 20:41:55 ntalex Exp $
+// Created:     29.10.2006
+// RCS-ID:      $Id: NewProjectDlg.h,v 1.6 2009/03/29 19:35:03 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
 
-#ifndef NEW_PROJECT_PROP_DLG_H
-#define NEW_PROJECT_PROP_DLG_H
+#ifndef NEW_PROJECT_DLG_H
+#define NEW_PROJECT_DLG_H
 
 #include <wx/wx.h>
 #include <wxVillaLib/PropDlg.h>
@@ -17,8 +17,11 @@
 #include "Menu.h"
 
 class NewProjectDlg: public wxPropDlg {
-  public:
-	NewProjectDlg(wxWindow *parent);
+public:
+	/**
+	 * Constructor. 
+	 */
+	NewProjectDlg(wxWindow *parent, bool create = true);
 	virtual ~NewProjectDlg() {}
 	
 	wxString GetLabel();
@@ -28,9 +31,10 @@ class NewProjectDlg: public wxPropDlg {
 	AudioFormat GetAudioFormat();
 	AspectRatio GetAspectRatio();
 	
-  protected:
-	void CreatePropPanel(wxSizer* sizer);
-	bool SetValues() { return true; }
+protected:
+	int propIndex;
+	virtual void CreatePropPanel(wxSizer* sizer);
+	virtual bool SetValues() { return true; }
 };
 
-#endif // NEW_PROJECT_PROP_DLG_H
+#endif // NEW_PROJECT_DLG_H
