@@ -3,7 +3,7 @@
 // Purpose:     Progress of generation/burning dialog
 // Author:      Alex Thuering
 // Created:     14.08.2004
-// RCS-ID:      $Id: ProgressDlg.cpp,v 1.73 2009/03/04 20:09:57 ntalex Exp $
+// RCS-ID:      $Id: ProgressDlg.cpp,v 1.83 2009/08/02 19:56:58 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -20,10 +20,12 @@
 #include <wx/filename.h>
 #include <wx/file.h>
 #include <wx/regex.h>
+#include <wx/utils.h>
 #include <wxSVG/SVGDocument.h>
 #include <wxSVG/mediadec_ffmpeg.h>
 
 #define DATA_FILE(fname) wxFindDataFile(_T("data") + wxString(wxFILE_SEP_PATH) + fname)
+#define TMP_ISO wxT("dvd-out.iso")
 
 ////////////////////////// Process Execute ///////////////////////////////////
 
@@ -206,7 +208,7 @@ BEGIN_EVENT_TABLE(ProgressDlg, wxDialog)
   EVT_BUTTON(ICONIZE_BT_ID, ProgressDlg::OnMinimize)
 END_EVENT_TABLE()
 
-ProgressDlg::ProgressDlg(wxWindow* parent, bool autoStart):
+ProgressDlg::ProgressDlg(wxWindow* parent, Cache* cache, bool autoStart):
 		wxDialog(parent, -1, wxEmptyString, wxDefaultPosition, wxDefaultSize,
 		wxCAPTION|wxRESIZE_BORDER|wxSYSTEM_MENU|wxTHICK_FRAME) {
     // begin wxGlade: ProgressDlg::ProgressDlg
@@ -224,6 +226,7 @@ ProgressDlg::ProgressDlg(wxWindow* parent, bool autoStart):
     // end wxGlade
     m_cancel = false;
     m_subStepCount = 0;
+    m_cache = cache;
     m_autoStart = autoStart;
 }
 
@@ -342,14 +345,15 @@ bool ProgressDlg::IsCanceled() {
 	return m_cancel;
 }
 
-void ProgressDlg::Start(BurnDlg* burnDlg, DVD* dvd) {
-	// disable window and redirect log messages into progress dilog details area
+bool ProgressDlg::Start(BurnDlg* burnDlg, DVD* dvd) {
+	// disable window and redirect log messages into progress dialog details area
 	m_winDisabler = new wxWindowDisabler(this);
 	wxLog* previousLog = wxLog::SetActiveTarget(new ProgressDlgLog(this));
 	// show dialog
 	Show();
 	// start
 	Run(burnDlg, dvd);
+	bool isOk = !IsCanceled();
 	// end
 	End();
 	// restore log and enable window
@@ -361,9 +365,11 @@ void ProgressDlg::Start(BurnDlg* burnDlg, DVD* dvd) {
 		Close(true);
     else
     	ShowModal();
+	
+	return isOk;
 }
 
-int GetMenuSubSteps(wxArrayPtrVoid& menuVobs, DVD* dvd, wxString tmpDVDDir) {
+int ProgressDlg::GetMenuSubSteps(wxArrayPtrVoid& menuVobs, DVD* dvd, wxString dvdTmpDir) {
 	int menuSubSteps = 0;
 	for (int tsi = -1; tsi < (int) dvd->GetTitlesets().Count(); tsi++) {
 		PgcArray& pgcs = dvd->GetPgcArray(tsi, true);
@@ -371,7 +377,7 @@ int GetMenuSubSteps(wxArrayPtrVoid& menuVobs, DVD* dvd, wxString tmpDVDDir) {
 			Pgc* pgc = pgcs[pgci];
 			if (pgc->GetVobs().Count() == 0)
 				continue;
-			wxString menuFile = tmpDVDDir + wxString::Format(_T("menu%d-%d.mpg"), tsi + 1, pgci);
+			wxString menuFile = dvdTmpDir + wxString::Format(_T("menu%d-%d.mpg"), tsi + 1, pgci);
 			Vob* vob = pgc->GetVobs()[0];
 			vob->SetTmpFilename(menuFile);
 			menuSubSteps += 10; // perpare
@@ -388,7 +394,7 @@ int GetMenuSubSteps(wxArrayPtrVoid& menuVobs, DVD* dvd, wxString tmpDVDDir) {
 	return menuSubSteps;
 }
 
-int GetTitleSubSteps(wxArrayPtrVoid& titleVobs, DVD* dvd, wxString tmpDVDDir) {
+int ProgressDlg::GetTitleSubSteps(wxArrayPtrVoid& titleVobs, DVD* dvd, Cache* cache) {
 	int titleSubSteps = 0;
 	for (int tsi = 0; tsi < (int) dvd->GetTitlesets().Count(); tsi++) {
 		Titleset* ts = dvd->GetTitlesets()[tsi];
@@ -397,15 +403,17 @@ int GetTitleSubSteps(wxArrayPtrVoid& titleVobs, DVD* dvd, wxString tmpDVDDir) {
 			for (int vobi = 0; vobi < (int) pgc->GetVobs().Count(); vobi++) {
 				Vob* vob = pgc->GetVobs()[vobi];
 				vob->SetTmpFilename(_T(""));
-				if (vob->GetSlideshow())
+				if (vob->GetSlideshow()
+						|| vob->GetDoNotTranscode() && vob->GetAudioFilenames().Count() == 0)
 					continue;
-				else if (vob->GetFilename().length()) {
-					if (vob->GetDoNotTranscode() && vob->GetAudioFilenames().Count() == 0)
-						continue;
-					titleSubSteps += 200;
+				wxString cacheFile = cache->Find(vob, dvd);
+				if (cacheFile.length() > 0 && wxFileExists(cacheFile)) {
+					AddDetailMsg(wxString::Format(_("Found transcoded file in cache for '%s'"),
+							vob->GetFilename().c_str()));
+					vob->SetTmpFilename(cacheFile);
+					continue; // file in cahe do not need to transcode
 				}
-				vob->SetTmpFilename(tmpDVDDir
-						+ wxString::Format(_T("title%d-%d-%d.vob"), tsi, pgci, vobi));
+				titleSubSteps += 200;
 				titleVobs.Add(vob);
 			}
 		}
@@ -413,7 +421,7 @@ int GetTitleSubSteps(wxArrayPtrVoid& titleVobs, DVD* dvd, wxString tmpDVDDir) {
 	return titleSubSteps;
 }
 
-int GetSlideshowSubSteps(wxArrayPtrVoid& slideshowVobs, DVD* dvd, wxString tmpDVDDir) {
+int ProgressDlg::GetSlideshowSubSteps(wxArrayPtrVoid& slideshowVobs, DVD* dvd, wxString dvdTmpDir) {
 	int slideshowSubSteps = 0;
 	for (int tsi = 0; tsi < (int) dvd->GetTitlesets().Count(); tsi++) {
 		Titleset* ts = dvd->GetTitlesets()[tsi];
@@ -423,7 +431,7 @@ int GetSlideshowSubSteps(wxArrayPtrVoid& slideshowVobs, DVD* dvd, wxString tmpDV
 				Vob* vob = pgc->GetVobs()[vobi];
 				if (vob->GetSlideshow()) {
 					slideshowSubSteps += 10 * vob->GetSlideshow()->Count();
-					vob->SetTmpFilename(tmpDVDDir
+					vob->SetTmpFilename(dvdTmpDir
 							+ wxString::Format(_T("title%d-%d-%d.vob"), tsi, pgci, vobi));
 					slideshowVobs.Add(vob);
 				}
@@ -433,7 +441,7 @@ int GetSlideshowSubSteps(wxArrayPtrVoid& slideshowVobs, DVD* dvd, wxString tmpDV
 	return slideshowSubSteps;
 }
 
-int GetSubtitleSubSteps(wxArrayPtrVoid& subtitleVobs, DVD* dvd, wxString tmpDVDDir) {
+int ProgressDlg::GetSubtitleSubSteps(wxArrayPtrVoid& subtitleVobs, DVD* dvd, wxString dvdTmpDir) {
 	int subtitleSubSteps = 0;
 	for (int tsi = 0; tsi < (int) dvd->GetTitlesets().Count(); tsi++) {
 		Titleset* ts = dvd->GetTitlesets()[tsi];
@@ -443,7 +451,7 @@ int GetSubtitleSubSteps(wxArrayPtrVoid& subtitleVobs, DVD* dvd, wxString tmpDVDD
 				Vob* vob = pgc->GetVobs()[vobi];
 				if (vob->GetSubtitles().Count()) {
 					subtitleSubSteps += vob->GetSize(dvd) * vob->GetSubtitles().Count();
-					vob->SetTmpFilename(tmpDVDDir
+					vob->SetTmpFilename(dvdTmpDir
 							+ wxString::Format(_T("title%d-%d-%d.vob"), tsi, pgci, vobi));
 					subtitleVobs.Add(vob);
 				}
@@ -459,30 +467,40 @@ void ProgressDlg::Run(BurnDlg* burnDlg, DVD* dvd) {
 
 	// print version
 	AddDetailMsg(wxT("DVDStyler v") + APP_VERSION);
+	AddDetailMsg(wxGetOsDescription());
 	AddDetailMsg(wxFfmpegMediaEncoder::GetBackendVersion());
+	
+	// prepare
+	AddSummaryMsg(_("Prepare"));
 
 	wxString tmpDir = burnDlg->GetTempDir();
 	if (tmpDir.Last() != wxFILE_SEP_PATH)
 		tmpDir += wxFILE_SEP_PATH;
-	wxString tmpDVDDir = tmpDir + wxString(wxT("dvd")) + wxFILE_SEP_PATH;
-
+	wxString dvdOutDir = tmpDir + wxString(wxT("dvd-out")) + wxFILE_SEP_PATH;
+	wxString dvdTmpDir = tmpDir + wxString(wxT("dvd-tmp")) + wxFILE_SEP_PATH;
+	
+	if (!CleanTemp(tmpDir, dvdTmpDir, dvdOutDir))
+		return;
+	
+	// check cache and calculate steps
+	AddDetailMsg(_("Search for transcoded files in cache"));
+	m_cache->BeginClean();
 	// menus
 	wxArrayPtrVoid menuVobs;
-	int menuSubSteps = GetMenuSubSteps(menuVobs, dvd, tmpDVDDir);
-
+	int menuSubSteps = GetMenuSubSteps(menuVobs, dvd, dvdTmpDir);
 	// titles
 	wxArrayPtrVoid titleVobs;
-	int titleSubSteps = GetTitleSubSteps(titleVobs, dvd, tmpDVDDir);
-
+	int titleSubSteps = GetTitleSubSteps(titleVobs, dvd, m_cache);
 	// slideshow
 	wxArrayPtrVoid slideshowVobs;
-	int slideshowSubSteps = GetSlideshowSubSteps(slideshowVobs, dvd, tmpDVDDir);
-
+	int slideshowSubSteps = GetSlideshowSubSteps(slideshowVobs, dvd, dvdTmpDir);
 	// subtitle
 	wxArrayPtrVoid subtitleVobs;
-	int subtitleSubSteps = GetSubtitleSubSteps(subtitleVobs, dvd, tmpDVDDir);
+	int subtitleSubSteps = GetSubtitleSubSteps(subtitleVobs, dvd, dvdTmpDir);
+	// remove unused files from cache
+	m_cache->EndClean();
 
-	// BEGIN
+	// calculate step count
 	int stepCount = 1;
 	if (menuVobs.Count() > 0)
 		stepCount++;
@@ -503,26 +521,26 @@ void ProgressDlg::Run(BurnDlg* burnDlg, DVD* dvd) {
 			stepCount++;
 	}
 	SetSteps(stepCount);
-
-	if (!Prepare(tmpDir, tmpDVDDir)
-			|| !GenerateMenus(menuVobs, menuSubSteps, dvd)
+	
+	// Start generation
+	if (!GenerateMenus(menuVobs, menuSubSteps, dvd)
 			|| !Transcode(titleVobs, titleSubSteps, dvd)
 			|| !GenerateSlideshow(slideshowVobs, slideshowSubSteps, dvd)
 			|| !MultiplexSubtitles(subtitleVobs, subtitleSubSteps, dvd)
-			|| !GenerateDvdFilesystem(dvd, tmpDVDDir, tmpDir)
-			|| !Preview(burnDlg, tmpDVDDir)
-			|| !CreateIsoImage(burnDlg, dvd, tmpDVDDir, tmpDir)
+			|| !GenerateDvdFilesystem(dvd, dvdTmpDir, dvdOutDir, tmpDir)
+			|| !Preview(burnDlg, dvdOutDir)
+			|| !CreateIsoImage(burnDlg, dvd, dvdOutDir, tmpDir)
 			|| !AddEccData(burnDlg, tmpDir)
 			|| !FormatDvd(burnDlg)
-			|| !BurnDvd(burnDlg, dvd, tmpDVDDir, tmpDir))
+			|| !BurnDvd(burnDlg, dvd, dvdOutDir, tmpDir))
 		return;
 
 	if (IsCanceled())
 		return;
 
-	// remove tmp dir
+	// clear temp directory
 	if ((burnDlg->DoCreateIso() || burnDlg->DoBurn()) && s_config.GetRemoveTempFiles())
-		DeleteTempDir(tmpDir);
+		DeleteTempFiles(tmpDir, dvdTmpDir, dvdOutDir);
 
 	if (burnDlg->DoBurn())
 		AddSummaryMsg(_("Burning was successful."), wxEmptyString, wxColour(0, 128, 0));
@@ -534,20 +552,21 @@ void ProgressDlg::Run(BurnDlg* burnDlg, DVD* dvd) {
 	s_config.SetKey(s_config.GetKey() + 1);
 }
 
-bool ProgressDlg::Prepare(const wxString& tmpDir, const wxString& tmpDVDDir) {
-	AddSummaryMsg(_("Prepare"));
-	AddDetailMsg(_("Cleaning temporary directory"));
-	if (wxDir::Exists(tmpDir) && !DeleteTempDir(tmpDir)) {
-		Failed(wxString::Format(_("Can't remove directory '%s'"), tmpDir.c_str()));
+bool ProgressDlg::CleanTemp(const wxString& tmpDir, const wxString& dvdTmpDir, const wxString& dvdOutDir) {
+	if (wxDir::Exists(tmpDir) && !DeleteTempFiles(tmpDir, dvdTmpDir, dvdOutDir))
 		return false;
-	}
-	// create tmp dir
+	
+	// create temporary directories
 	if (!wxDir::Exists(tmpDir) && !wxMkdir(tmpDir)) {
 		Failed(wxString::Format(_("Can't create directory '%s'"), tmpDir.c_str()));
 		return false;
 	}
-	if (!wxDir::Exists(tmpDVDDir) && !wxMkdir(tmpDVDDir)) {
-		Failed(wxString::Format(_("Can't create directory '%s'"), tmpDVDDir.c_str()));
+	if (!wxDir::Exists(dvdTmpDir) && !wxMkdir(dvdTmpDir)) {
+		Failed(wxString::Format(_("Can't create directory '%s'"), dvdTmpDir.c_str()));
+		return false;
+	}
+	if (!wxDir::Exists(dvdOutDir) && !wxMkdir(dvdOutDir)) {
+		Failed(wxString::Format(_("Can't create directory '%s'"), dvdOutDir.c_str()));
 		return false;
 	}
 	return true;
@@ -581,10 +600,13 @@ bool ProgressDlg::Transcode(wxArrayPtrVoid& titleVobs, int titleSubSteps, DVD* d
 	SetSubSteps(titleSubSteps);
 	for (int i = 0; i < (int) titleVobs.Count(); i++) {
 		Vob* vob = (Vob*) titleVobs[i];
-		if (vob->GetFilename().length()) {
-			if (!Transcode(vob->GetStreams(), vob->GetFilename(), vob->GetAudioFilenames(),
-					vob->GetTmpFilename(), dvd->GetVideoBitrate()))
-				return false;
+		wxString cacheFile = m_cache->Add(vob, dvd);
+		AddDetailMsg(wxT("Add file to cache:") + cacheFile);
+		vob->SetTmpFilename(cacheFile);
+		if (!Transcode(vob->GetStreams(), vob->GetFilename(), vob->GetAudioFilenames(),
+				vob->GetTmpFilename(), dvd->GetVideoBitrate())) {
+			wxRemoveFile(vob->GetTmpFilename());
+			return false;
 		}
 	}
 	IncStep();
@@ -698,16 +720,17 @@ bool ProgressDlg::MultiplexSubtitles(const wxString& vobFile, TextSub* textSub,
 	return true;
 }
 
-bool ProgressDlg::GenerateDvdFilesystem(DVD* dvd, const wxString& tmpDVDDir, const wxString& tmpDir) {
+bool ProgressDlg::GenerateDvdFilesystem(DVD* dvd, const wxString& dvdTmpDir, const wxString& dvdOutDir,
+		const wxString& tmpDir) {
 	if (IsCanceled())
 		return false;
-	wxString dvdauthFile = tmpDVDDir + _T("dvdauthor.xml");
+	wxString dvdauthFile = dvdTmpDir + _T("dvdauthor.xml");
 	dvd->SaveDVDAuthor(dvdauthFile);
 	AddSummaryMsg(_("Generating DVD"));
 	SetSubSteps(300);
 	wxString cmd = s_config.GetDvdauthorCmd();
 	cmd.Replace(_T("$FILE_CONF"), dvdauthFile);
-	cmd.Replace(_T("$DIR"), tmpDVDDir.Mid(0, tmpDVDDir.length() - 1));
+	cmd.Replace(_T("$DIR"), dvdOutDir.Mid(0, dvdOutDir.length() - 1));
 	DVDAuthorExecute dvdauthorExec(this, dvd->GetSize(true) / 1024);
 	if (!dvdauthorExec.Execute(cmd)) {
 		Failed();
@@ -715,22 +738,17 @@ bool ProgressDlg::GenerateDvdFilesystem(DVD* dvd, const wxString& tmpDVDDir, con
 	}
 	// remove temp files
 	if (s_config.GetRemoveTempFiles()) {
-		wxDir d(tmpDir + wxString(wxT("dvd")));
+		wxDir d(dvdTmpDir);
 		wxString fname;
-		while (d.GetFirst(&fname, _T("*.vob"), wxDIR_FILES))
-			if (!DeleteFile(tmpDVDDir + fname))
+		while (d.GetFirst(&fname, wxEmptyString, wxDIR_FILES))
+			if (!DeleteFile(dvdTmpDir + fname))
 				return false;
-		while (d.GetFirst(&fname, _T("menu*"), wxDIR_FILES))
-			if (!DeleteFile(tmpDVDDir + fname))
-				return false;
-		if (!DeleteFile(dvdauthFile))
-			return false;
 	}
 	IncStep();
 	return true;
 }
 
-bool ProgressDlg::Preview(BurnDlg* burnDlg, const wxString& tmpDVDDir) {
+bool ProgressDlg::Preview(BurnDlg* burnDlg, const wxString& dvdOutDir) {
 	if (!burnDlg->DoPreview())
 		return true;
 	if (IsCanceled())
@@ -738,21 +756,36 @@ bool ProgressDlg::Preview(BurnDlg* burnDlg, const wxString& tmpDVDDir) {
 	AddSummaryMsg(_("Start preview"));
 	wxString cmd = s_config.GetPreviewCmd();
 	if (cmd.length() > 0) {
-		if (cmd.Find(wxT("$DIR")) < 0) {
-			if (cmd[0] != wxT('"'))
-				cmd = wxT('"') + cmd + wxT('"');
-			cmd += wxT(" \"dvd:/$DIR\"");
+		if (cmd == wxT("wmplayer")) {
+			cmd = wxT("start wmplayer \"$DIR\\VIDEO_TS\\VIDEO_TS.VOB\"");
+			cmd.Replace(wxT("$DIR"), dvdOutDir.Mid(0, dvdOutDir.length() - 1));
+			if (!wxShell(cmd)) {
+				wxString msg = _("Starting of DVD player is failed. \
+Please check the path to the DVD player in the 'Settings/Core/Preview command' \
+or open the following directory with your DVD player: ");
+				msg += dvdOutDir;
+				wxMessageBox(msg, _("Burn"), wxOK|wxICON_ERROR, this);
+			}
+		} else {
+			if (cmd.Find(wxT("$DIR")) < 0) {
+				if (cmd[0] != wxT('"'))
+					cmd = wxT('"') + cmd + wxT('"');
+				if (cmd.Find(wxT("wmplayer")) >= 0)
+					cmd += wxT(" \"$DIR\\VIDEO_TS\\VIDEO_TS.VOB\"");
+				else
+					cmd += wxT(" \"dvd:/$DIR\"");
+			}
+			cmd.Replace(wxT("$DIR"), dvdOutDir.Mid(0, dvdOutDir.length() - 1));
+			if (!Exec(cmd))
+				Failed();
 		}
-		cmd.Replace(wxT("$DIR"), tmpDVDDir.Mid(0, tmpDVDDir.length() - 1));
-		if (!Exec(cmd))
-			Failed();
 	} else {
 		wxString msg =
 			_("Unfortunately there is no DVD player specified in the DVDStyler settings. \
 Please set the path to the DVD player in the 'Settings/Core/Preview command' \
 or open the following directory with your DVD player: ");
-		msg += tmpDVDDir;
-		wxMessageBox(msg, _("Burn"), wxOK, this);
+		msg += dvdOutDir;
+		wxMessageBox(msg, _("Burn"), wxOK|wxICON_INFORMATION, this);
 	}
 	if (burnDlg->DoBurn() || burnDlg->DoCreateIso()) {
 		wxString msg = burnDlg->DoBurn() ? _("Do you want to burn this video to DVD?") : _("Do you want to create an iso image of this video?");
@@ -764,17 +797,42 @@ or open the following directory with your DVD player: ");
 	return true;
 }
 
-bool ProgressDlg::CreateIsoImage(BurnDlg* burnDlg, DVD* dvd, const wxString& tmpDVDDir,
+long GetFreeSpaceOn(wxString dir) {
+	wxDiskspaceSize_t pFree;
+	wxGetDiskSpace(dir, NULL, &pFree);
+	return (long) (pFree.ToDouble() / 1024);
+}
+
+bool ProgressDlg::CreateIsoImage(BurnDlg* burnDlg, DVD* dvd, const wxString& dvdOutDir,
 		const wxString& tmpDir) {
 	if (burnDlg->DoCreateIso() || (burnDlg->DoBurn() && burnDlg->DoAddECC())) {
 		if (IsCanceled())
 			return false;
+		wxString isoFile = burnDlg->DoCreateIso() ? burnDlg->GetIsoFile() : tmpDir + TMP_ISO;
+		// check if there is enough space
+		long size = 0;
+		wxString cmd = s_config.GetIsoSizeCmd();
+		cmd.Replace(_T("$DIR"), dvdOutDir.Mid(0, dvdOutDir.length() - 1));
+#ifdef __WXMSW__
+		cmd = wxGetAppPath() + wxString(wxFILE_SEP_PATH) + _T("..") + wxString(wxFILE_SEP_PATH) + _T("dvdauthor") + wxString(wxFILE_SEP_PATH) + cmd;
+#endif
+		wxArrayString output;
+		wxExecute(cmd, output, wxEXEC_SYNC | wxEXEC_NODISABLE);
+		if (output.Count() > 0 && output[0].length() > 0) {
+			output[0].ToLong(&size);
+			size = (size + 254)*2;
+		}
+		long freeSpace = GetFreeSpaceOn(wxFileName(isoFile).GetPath());
+		if (size > freeSpace && freeSpace == GetFreeSpaceOn(m_cache->GetTempDir())) {
+			AddDetailMsg(_("There is not enough space to store ISO: cache emptying."));
+			m_cache->Clear();
+		}
 		AddSummaryMsg(_("Creating ISO image"));
 		SetSubSteps(100);
-		wxString cmd = s_config.GetIsoCmd();
+		cmd = s_config.GetIsoCmd();
 		cmd.Replace(_T("$VOL_ID"), dvd->GetLabel());
-		cmd.Replace(_T("$DIR"), tmpDVDDir.Mid(0, tmpDVDDir.length() - 1));
-		cmd.Replace(_T("$FILE"), burnDlg->DoCreateIso() ? burnDlg->GetIsoFile() : tmpDir + wxT("dvd.iso"));
+		cmd.Replace(_T("$DIR"), dvdOutDir.Mid(0, dvdOutDir.length() - 1));
+		cmd.Replace(_T("$FILE"), isoFile);
 		ProgressExecute exec(this, wxT(".*"));
 		if (!exec.Execute(cmd)) {
 			Failed();
@@ -786,14 +844,14 @@ bool ProgressDlg::CreateIsoImage(BurnDlg* burnDlg, DVD* dvd, const wxString& tmp
 }
 
 bool ProgressDlg::AddEccData(BurnDlg* burnDlg, const wxString& tmpDir) {
-	if (!burnDlg->DoAddECC())
+	if (!burnDlg->DoAddECC() || !burnDlg->DoCreateIso() && !burnDlg->DoBurn())
 		return true;
 	if (IsCanceled())
 		return false;
 	AddSummaryMsg(_("Adding ECC data"));
 	SetSubSteps(200);
 	wxString cmd = s_config.GetAddECCCmd();
-	cmd.Replace(_T("$FILE"), burnDlg->DoCreateIso() ? burnDlg->GetIsoFile() : tmpDir + wxT("dvd.iso"));
+	cmd.Replace(_T("$FILE"), burnDlg->DoCreateIso() ? burnDlg->GetIsoFile() : tmpDir + TMP_ISO);
 	ProgressExecute exec(this, wxT("(Preparing|Ecc).*"));
 	if (!exec.Execute(cmd)) {
 		Failed();
@@ -832,7 +890,7 @@ bool ProgressDlg::FormatDvd(BurnDlg* burnDlg) {
 	return true;
 }
 
-bool ProgressDlg::BurnDvd(BurnDlg* burnDlg, DVD* dvd, const wxString& tmpDVDDir,
+bool ProgressDlg::BurnDvd(BurnDlg* burnDlg, DVD* dvd, const wxString& dvdOutDir,
 		const wxString& tmpDir) {
 	if (!burnDlg->DoBurn())
 		return true;
@@ -871,10 +929,10 @@ bool ProgressDlg::BurnDvd(BurnDlg* burnDlg, DVD* dvd, const wxString& tmpDVDDir,
 	// check size
 	long size = 0;
 	if (burnDlg->DoAddECC()) {
-		size = wxFile(tmpDir + wxT("dvd.iso")).Length() / 2048; // size in 2048 blocks
+		size = wxFile(tmpDir + TMP_ISO).Length() / 2048; // size in 2048 blocks
 	} else {
 		cmd = s_config.GetIsoSizeCmd();
-		cmd.Replace(_T("$DIR"), tmpDVDDir.Mid(0, tmpDVDDir.length() - 1));
+		cmd.Replace(_T("$DIR"), dvdOutDir.Mid(0, dvdOutDir.length() - 1));
 #ifdef __WXMSW__
 		cmd = wxGetAppPath() + wxString(wxFILE_SEP_PATH) + _T("..") + wxString(wxFILE_SEP_PATH) + _T("dvdauthor") + wxString(wxFILE_SEP_PATH) + cmd;
 #endif
@@ -894,13 +952,13 @@ bool ProgressDlg::BurnDvd(BurnDlg* burnDlg, DVD* dvd, const wxString& tmpDVDDir,
 	// burn
 	if (burnDlg->DoAddECC()) {
 		cmd = s_config.GetBurnISOCmd();
-		cmd.Replace(_T("$FILE"), tmpDir + wxT("dvd.iso"));
+		cmd.Replace(_T("$FILE"), tmpDir + TMP_ISO);
 		// get iso size in sectors
-		long size = wxFile(tmpDir + wxT("dvd.iso")).Length() / 2048;
+		long size = wxFile(tmpDir + TMP_ISO).Length() / 2048;
 		cmd.Replace(_T("$SIZE"), wxString::Format(wxT("%d"),size));
 	} else {
 		cmd = s_config.GetBurnCmd();
-		cmd.Replace(_T("$DIR"), tmpDVDDir.Mid(0, tmpDVDDir.length() - 1));
+		cmd.Replace(_T("$DIR"), dvdOutDir.Mid(0, dvdOutDir.length() - 1));
 	}
 	cmd.Replace(_T("$VOL_ID"), dvd->GetLabel());
 	cmd.Replace(_T("$DEV"), burnDlg->GetDevice());
@@ -929,7 +987,7 @@ bool ProgressDlg::Transcode(const StreamArray& streams, const wxString& mpegFile
 		&& streams[1]->GetDestinationFormat() == afNONE;
 	if (disableAudio)
 		wxLogMessage(wxT("disableAudio"));
-	wxFfmpegMediaTranscoder transcoder;
+	wxFfmpegMediaTranscoder transcoder(s_config.GetThreadCount());
 	if (!transcoder.AddInputFile(mpegFile, false, disableAudio)) {
 		Failed(wxT("Error by transcoding of ") + mpegFile);
 		return false;
@@ -962,6 +1020,8 @@ bool ProgressDlg::Transcode(const StreamArray& streams, const wxString& mpegFile
 	}
 	if (s_config.GetUseMplex()) {
 		wxString videoFile = vobFile + wxT(".m2v");
+		if (wxFileExists(videoFile) && !wxRemoveFile(videoFile))
+			wxLogError(wxString::Format(_("Can't remove file '%s'"), videoFile.c_str()));
 		if (!transcoder.SetOutputFile(videoFile, videoFormat, afNONE, sfNONE,
 				videoBitrate, s_config.GetVbr(), s_config.GetAudioBitrate())) {
 			Failed(_("Error transcoding of ") + mpegFile);
@@ -973,19 +1033,28 @@ bool ProgressDlg::Transcode(const StreamArray& streams, const wxString& mpegFile
 				continue;
 			wxString audioFile = vobFile + wxString::Format(wxT(".audio%d"), audioIdx);
 			audioFiles.Add(audioFile);
+			if (wxFileExists(audioFile) && !wxRemoveFile(audioFile))
+				wxLogError(wxString::Format(_("Can't remove file '%s'"), audioFile.c_str()));
 			if (!transcoder.SetOutputFile(audioFile, vfNONE, (AudioFormat) audioFormats[audioIdx], sfNONE,
-							videoBitrate, s_config.GetVbr(), s_config.GetAudioBitrate())) {
+						videoBitrate, s_config.GetVbr(), s_config.GetAudioBitrate())) {
 				Failed(_("Error transcoding of ") + mpegFile);
 				return false;
 			}
 		}
 		if (!transcoder.Run(m_cancel)) {
+			wxRemoveFile(videoFile);
+			for (unsigned int audioIdx = 0; audioIdx < audioFiles.GetCount(); audioIdx++)
+				wxRemoveFile(audioFiles[audioIdx]);
 			Failed(_("Error transcoding of ") + mpegFile);
 			return false;
 		}
 		SetSubStep(subStep+150);
-		if (!Multiplex(videoFile, audioFiles, vobFile))
+		if (!Multiplex(videoFile, audioFiles, vobFile)) {
+			wxRemoveFile(videoFile);
+			for (unsigned int audioIdx = 0; audioIdx < audioFiles.GetCount(); audioIdx++)
+				wxRemoveFile(audioFiles[audioIdx]);
 			return false;
+		}
 		// remove temp files
 		if (s_config.GetRemoveTempFiles()) {
 			if (!DeleteFile(videoFile))
@@ -999,6 +1068,7 @@ bool ProgressDlg::Transcode(const StreamArray& streams, const wxString& mpegFile
 		if (!transcoder.SetOutputFile(vobFile, videoFormat, audioFormats, subtitleFormats,
 				videoBitrate, s_config.GetVbr(), s_config.GetAudioBitrate())
 				|| !transcoder.Run(m_cancel)) {
+			wxRemoveFile(vobFile);
 			Failed(_("Error transcoding of ") + mpegFile);
 			return false;
 		}
@@ -1109,7 +1179,7 @@ bool ProgressDlg::GenerateMenu(Menu* menu, const wxString& menuFile,
 				}
 			}
 		}
-		wxFfmpegMediaEncoder ffmpeg;
+		wxFfmpegMediaEncoder ffmpeg(s_config.GetThreadCount());
 		if (!ffmpeg.BeginEncode(audioFile.length() ? m2vFile : mpegFile, menu->GetVideoFormat(),
 				audioFile.length() ? afNONE : audioFormat,
 				menu->GetAspectRatio(), s_config.GetMenuVideoBitrate())
@@ -1179,7 +1249,7 @@ bool ProgressDlg::GenerateSlideshow(Slideshow* slideshow, const wxString& vobFil
 		return false;
 	AddDetailMsg(_("Generating slideshow"));
 
-	wxFfmpegMediaEncoder ffmpeg;
+	wxFfmpegMediaEncoder ffmpeg(s_config.GetThreadCount());
 	if (!ffmpeg.BeginEncode(vobFile, slideshow->GetVideoFormat(), audioFormat, slideshow->GetAspectRatio(),
 			s_config.GetSlideshowVideoBitrate())) {
 		Failed(_("Error creation of slideshow"));
@@ -1227,32 +1297,14 @@ bool ProgressDlg::DeleteDir(wxString dir, bool subdirs) {
 	return true;
 }
 
-bool ProgressDlg::DeleteTempDir(wxString dir) {
-	if (dir.Last() != wxFILE_SEP_PATH)
-		dir += wxFILE_SEP_PATH;
-	if (wxDirExists(dir + wxT("dvd"))) {
-		wxDir d(dir + wxT("dvd"));
-		wxString fname;
-		d.GetFirst(&fname, wxEmptyString, wxDIR_DIRS);
-		while (1) {
-			if (fname.Lower() == _T("video_ts") || fname.Lower() == _T("audio_ts"))
-				if (!DeleteDir(dir + wxT("dvd") + wxFILE_SEP_PATH + fname, false))
-					return false;
-			if (!d.GetNext(&fname))
-				break;
-		}
-		d.GetFirst(&fname, wxEmptyString, wxDIR_FILES | wxDIR_HIDDEN);
-		while (1) {
-			if (fname.Mid(0, 4).Lower() == _T("menu") || fname.Mid(0, 5).Lower() == _T("title"))
-				if (!DeleteFile(dir + wxT("dvd") + wxFILE_SEP_PATH + fname))
-					return false;
-			if (!d.GetNext(&fname))
-				break;
-		}
-		d.Open(wxGetHomeDir());
-		wxRmdir(dir + wxT("dvd"));
-	}
-	if (wxFileExists(dir + wxT("dvd.iso")) && !wxRemoveFile(dir + wxT("dvd.iso")))
+bool ProgressDlg::DeleteTempFiles(const wxString& tmpDir, const wxString& dvdTmpDir,
+		const wxString& dvdOutDir) {
+	AddDetailMsg(_("Cleaning temporary directory"));
+	if (wxDirExists(dvdTmpDir) && !DeleteDir(dvdTmpDir, false))
+		return false;
+	if (wxDirExists(dvdOutDir) && !DeleteDir(dvdOutDir, true))
+		return false;
+	if (wxFileExists(tmpDir + TMP_ISO) && !DeleteFile(tmpDir + TMP_ISO))
 		return false;
 	return true;
 }
