@@ -3,7 +3,7 @@
 // Purpose:     FFMPEG Media Transcoder
 // Author:      Alex Thuering
 // Created:     26.04.2008
-// RCS-ID:      $Id: mediatrc_ffmpeg.cpp,v 1.13 2009/01/25 12:42:49 ntalex Exp $
+// RCS-ID:      $Id: mediatrc_ffmpeg.cpp,v 1.20 2009/07/11 17:04:45 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -75,7 +75,7 @@ typedef struct AVOutputStream {
 	ReSampleContext *resample; /* for audio resampling */
 	int reformat_pair;
 	//AVAudioConvert *reformat_ctx;
-	AVFifoBuffer fifo; /* for compression: one audio fifo per codec */
+	AVFifoBuffer* fifo; /* for compression: one audio fifo per codec */
 	FILE *logfile;
 } AVOutputStream;
 
@@ -126,7 +126,7 @@ double psnr(double d) {
 }
 
 void av_log_wxLog_callback(void* ptr, int level, const char* fmt, va_list vl) {
-	if (level>av_log_level)
+	if (level>av_log_get_level())
 		return;
 	static wxString buf;
 	char s[1024];
@@ -138,7 +138,7 @@ void av_log_wxLog_callback(void* ptr, int level, const char* fmt, va_list vl) {
 	}
 }
 
-wxFfmpegMediaTranscoder::wxFfmpegMediaTranscoder() {
+wxFfmpegMediaTranscoder::wxFfmpegMediaTranscoder(int threadCount) {
 	nb_input_files = 0;
 	nb_output_files = 0;
 	nb_ocodecs = 0;
@@ -188,6 +188,7 @@ wxFfmpegMediaTranscoder::wxFfmpegMediaTranscoder() {
 	audio_language = NULL;
 
 	subtitle_language = NULL;
+	subtitle_codec_tag = 0;
 
 	mux_preload= 0.5;
 	mux_max_delay= 0.7;
@@ -196,12 +197,8 @@ wxFfmpegMediaTranscoder::wxFfmpegMediaTranscoder() {
 	start_time = 0;
 	rec_timestamp = 0;
 	input_ts_offset = 0;
-	str_title = NULL;
-	str_author = NULL;
-	str_copyright = NULL;
-	str_comment = NULL;
-	str_genre = NULL;
-	str_album = NULL;
+	metadata_count = 0;
+	metadata = NULL;
 	do_psnr = 0;
 	do_pass = 0;
 	pass_logfilename_prefix = "ffmpeg2pass";
@@ -224,9 +221,8 @@ wxFfmpegMediaTranscoder::wxFfmpegMediaTranscoder() {
 	audio_volume = 256;
 
 	exit_on_error = 0;
-	using_vhook = 0;
 	verbose = 1;
-	thread_count= 1;
+	thread_count = threadCount;
 	q_pressed = 0;
 	video_size = 0;
 	audio_size = 0;
@@ -245,15 +241,12 @@ wxFfmpegMediaTranscoder::wxFfmpegMediaTranscoder() {
 	video_bitstream_filters = NULL;
 	audio_bitstream_filters = NULL;
 	subtitle_bitstream_filters = NULL;
-#if LIBAVUTIL_VERSION_INT < (50<<16)
 	av_log_set_callback(&av_log_wxLog_callback);
-#else
-	av_vlog = av_log_wxLog_callback;
-#endif
 }
 
 wxFfmpegMediaTranscoder::~wxFfmpegMediaTranscoder() {
 	End();
+	av_log_set_callback(&av_log_default_callback);
 }
 
 bool wxFfmpegMediaTranscoder::AddInputFile(const wxString& fileName,
@@ -263,11 +256,11 @@ bool wxFfmpegMediaTranscoder::AddInputFile(const wxString& fileName,
 	int64_t timestamp;
 
 	/* get default parameters from command line */
-	ic = av_alloc_format_context();
+	ic = avformat_alloc_context();
+	ic->flags |= AVFMT_FLAG_NONBLOCK;
 
 	// open the input file with generic libav function
-	err = av_open_input_file(&ic, (const char*) fileName.mb_str(), NULL, 0, 
-	NULL);
+	err = av_open_input_file(&ic, (const char*) fileName.mb_str(), NULL, 0, NULL);
 	if (err < 0) {
 		PrintError(fileName, err);
 		return false;
@@ -406,7 +399,7 @@ bool wxFfmpegMediaTranscoder::SetOutputFile(const wxString& fileName,
 		return false;
 	}
 
-	AVFormatContext *oc = av_alloc_format_context();
+	AVFormatContext *oc = avformat_alloc_context();
 
 	oc->oformat = outputFormat;
 	ff_strlcpy(oc->filename, (const char*) fileName.mb_str(),
@@ -418,51 +411,22 @@ bool wxFfmpegMediaTranscoder::SetOutputFile(const wxString& fileName,
 		return false;
 	}
 	// add video, audio and subtitle streams
-	unsigned int afIdx = 0;
-	unsigned int sfIdx = 0;
-	for (int inputFileIdx = 0; inputFileIdx < nb_input_files; inputFileIdx++) {
-		AVFormatContext* ic = input_files[inputFileIdx];
-		for (int si=0; si<(int)ic->nb_streams; si++) {
-			switch (ic->streams[si]->codec->codec_type) {
-			case CODEC_TYPE_VIDEO:
-				if (videoFormat != vfNONE && !AddVideoStream(oc, videoFormat, videoBitrate, vbr))
-					return false;
-				break;
-			case CODEC_TYPE_AUDIO:
-				if (audioFormats.GetCount() > 0 && audioFormats[afIdx] != afNONE
-						&& !AddAudioStream(oc, (AudioFormat) audioFormats[afIdx], audioBitrate))
-					return false;
-				if (afIdx < audioFormats.GetCount() - 1)
-					afIdx++;
-				break;
-			case CODEC_TYPE_SUBTITLE:
-				if (subtitleFormats.GetCount() > 0 && subtitleFormats[sfIdx]
-						!= sfNONE && !AddSubtitleStream(oc,
-						(SubtitleFormat) subtitleFormats[sfIdx]))
-					return false;
-				if (sfIdx < subtitleFormats.GetCount() - 1)
-					sfIdx++;
-				break;
-			default:
-				break;
-			}
-		}
-	}
+	if (videoFormat != vfNONE && !AddVideoStream(oc, videoFormat, videoBitrate, vbr))
+		return false;
+	for (unsigned int afIdx = 0; afIdx < audioFormats.GetCount(); afIdx++)
+		if (audioFormats[afIdx] != afNONE && !AddAudioStream(oc, (AudioFormat) audioFormats[afIdx], audioBitrate))
+			return false;
+	for (unsigned int sfIdx = 0; sfIdx < subtitleFormats.GetCount(); sfIdx++)
+		if (subtitleFormats[sfIdx] != sfNONE && !AddSubtitleStream(oc, (SubtitleFormat) subtitleFormats[sfIdx]))
+			return false;
 
 	oc->timestamp = rec_timestamp;
 
-	if (str_title)
-		ff_strlcpy(oc->title, str_title, sizeof(oc->title));
-	if (str_author)
-		ff_strlcpy(oc->author, str_author, sizeof(oc->author));
-	if (str_copyright)
-		ff_strlcpy(oc->copyright, str_copyright, sizeof(oc->copyright));
-	if (str_comment)
-		ff_strlcpy(oc->comment, str_comment, sizeof(oc->comment));
-	if (str_album)
-		ff_strlcpy(oc->album, str_album, sizeof(oc->album));
-	if (str_genre)
-		ff_strlcpy(oc->genre, str_genre, sizeof(oc->genre));
+	for(; metadata_count>0; metadata_count--){
+		av_metadata_set(&oc->metadata, metadata[metadata_count-1].key,
+				metadata[metadata_count-1].value);
+	}
+	av_metadata_conv(oc, oc->oformat->metadata_conv, NULL);
 
 	output_files[nb_output_files++] = oc;
 
@@ -495,6 +459,7 @@ bool wxFfmpegMediaTranscoder::SetOutputFile(const wxString& fileName,
 	oc->preload= (int)(mux_preload*AV_TIME_BASE);
 	oc->max_delay= (int)(mux_max_delay*AV_TIME_BASE);
 	oc->loop_output = loop_output;
+	oc->flags |= AVFMT_FLAG_NONBLOCK;
 
 	return true;
 }
@@ -740,7 +705,7 @@ bool wxFfmpegMediaTranscoder::AddAudioStream(AVFormatContext *oc, AudioFormat au
 	audio_enc->sample_rate = audio_sample_rate;
 	audio_enc->time_base= (AVRational) {1, audio_sample_rate};
 	if (audio_language) {
-		ff_strlcpy(st->language, audio_language, sizeof(st->language));
+		av_metadata_set(&st->metadata, "language", audio_language);
 		av_free(audio_language);
 		audio_language = NULL;
 	}
@@ -769,6 +734,10 @@ bool wxFfmpegMediaTranscoder::AddSubtitleStream(AVFormatContext *oc,
 
 	subtitle_enc = st->codec;
 	subtitle_enc->codec_type = CODEC_TYPE_SUBTITLE;
+	
+	if(subtitle_codec_tag)
+		subtitle_enc->codec_tag= subtitle_codec_tag;
+	
 	if (subtitleFormat == sfCOPY) {
 		st->stream_copy = 1;
 	} else {
@@ -778,7 +747,7 @@ bool wxFfmpegMediaTranscoder::AddSubtitleStream(AVFormatContext *oc,
 	nb_ocodecs++;
 
 	if (subtitle_language) {
-		ff_strlcpy(st->language, subtitle_language, sizeof(st->language));
+		av_metadata_set(&st->metadata, "language", subtitle_language);
 		av_free(subtitle_language);
 		subtitle_language = NULL;
 	}
@@ -787,14 +756,17 @@ bool wxFfmpegMediaTranscoder::AddSubtitleStream(AVFormatContext *oc,
 }
 
 bool wxFfmpegMediaTranscoder::Run(bool& canceled) {
-	int ret, i, j, k, n, nb_istreams = 0, nb_ostreams = 0;
+	int ret = 0, i, j, k, n, nb_istreams = 0, nb_ostreams = 0;
 	AVFormatContext *is, *os;
 	AVCodecContext *codec, *icodec;
 	AVOutputStream *ost, **ost_table= NULL;
 	AVInputStream *ist, **ist_table= NULL;
+	wxString error;
 	int want_sdp = 1;
 	int bit_buffer_size = 1024*256;
 	uint8_t *bit_buffer= NULL;
+	uint8_t no_packet[MAX_FILES]={0};
+	int no_packet_count=0;
 	
 	if (canceled)
 		return false;
@@ -887,6 +859,7 @@ bool wxFfmpegMediaTranscoder::Run(bool& canceled) {
 		goto fail;
 	for (i=0; i<nb_ostreams; i++) {
 		ost = (AVOutputStream*) av_mallocz(sizeof(AVOutputStream));
+		ost->fifo = NULL;
 		if (!ost)
 			goto fail;
 		ost_table[i] = ost;
@@ -964,6 +937,7 @@ bool wxFfmpegMediaTranscoder::Run(bool& canceled) {
 
 	/* for each output stream, we compute the right encoding parameters */
 	for (i=0; i<nb_ostreams; i++) {
+		AVMetadataTag *lang;
 		ost = ost_table[i];
 		os = output_files[ost->file_index];
 		ist = ist_table[ost->source_index];
@@ -971,9 +945,10 @@ bool wxFfmpegMediaTranscoder::Run(bool& canceled) {
 		codec = ost->st->codec;
 		icodec = ist->st->codec;
 
-		if (!ost->st->language[0])
-			ff_strlcpy(ost->st->language, ist->st->language,
-					sizeof(ost->st->language));
+		if ((lang=av_metadata_get(ist->st->metadata, "language", NULL, 0))
+				&& !av_metadata_get(ost->st->metadata, "language", NULL, 0))
+			av_metadata_set(&ost->st->metadata, "language", lang->value);
+		 
 		
 		ost->st->disposition = ist->st->disposition;
 
@@ -993,10 +968,10 @@ bool wxFfmpegMediaTranscoder::Run(bool& canceled) {
 			codec->bit_rate = icodec->bit_rate;
 			codec->extradata= icodec->extradata;
 			codec->extradata_size= icodec->extradata_size;
-			if (av_q2d(icodec->time_base) > av_q2d(ist->st->time_base)
-					&& av_q2d(ist->st->time_base) < 1.0/1000)
+			if(av_q2d(icodec->time_base)*icodec->ticks_per_frame > av_q2d(ist->st->time_base) && av_q2d(ist->st->time_base) < 1.0/1000){
 				codec->time_base = icodec->time_base;
-			else
+				codec->time_base.num *= icodec->ticks_per_frame;
+			} else
 				codec->time_base = ist->st->time_base;
 			switch (codec->codec_type) {
 			case CODEC_TYPE_AUDIO:
@@ -1017,10 +992,6 @@ bool wxFfmpegMediaTranscoder::Run(bool& canceled) {
 					codec->block_align= 0;
 				break;
 			case CODEC_TYPE_VIDEO:
-				if (using_vhook) {
-					wxLogError(wxT("-vcodec copy and -vhook are incompatible (frames are not decoded)"));
-					return false;
-				}
 				codec->pix_fmt = icodec->pix_fmt;
 				codec->width = icodec->width;
 				codec->height = icodec->height;
@@ -1037,8 +1008,15 @@ bool wxFfmpegMediaTranscoder::Run(bool& canceled) {
 		} else {
 			switch (codec->codec_type) {
 			case CODEC_TYPE_AUDIO:
-				if (av_fifo_init(&ost->fifo, 1024))
+#if LIBAVUTIL_VERSION_INT > AV_VERSION_INT(49, 15, 0)
+				ost->fifo= av_fifo_alloc(1024);
+				if(!ost->fifo)
 					goto fail;
+#else
+				ost->fifo = (AVFifoBuffer*) av_mallocz(sizeof(AVFifoBuffer));
+				if (!ost->fifo || av_fifo_init(ost->fifo, 1024))
+					goto fail;
+#endif
 				ost->reformat_pair = MAKE_SFMT_PAIR(SAMPLE_FMT_NONE,SAMPLE_FMT_NONE);
 				ost->audio_resample = codec->sample_rate != icodec->sample_rate || audio_sync_method > 1;
 				icodec->request_channels = codec->channels;
@@ -1146,35 +1124,14 @@ bool wxFfmpegMediaTranscoder::Run(bool& canceled) {
 		}
 		if (codec->codec_type == CODEC_TYPE_VIDEO) {
 			int size= codec->width * codec->height;
-			bit_buffer_size= FFMAX(bit_buffer_size, 4*size);
+			bit_buffer_size= FFMAX(bit_buffer_size, 6*size + 200);
 		}
 	}
 
 	if (!bit_buffer)
 		bit_buffer = (uint8_t*) av_malloc(bit_buffer_size);
-	if (!bit_buffer)
+	if (!bit_buffer) {
 		goto fail;
-
-	// dump the file output parameters - cannot be done before in case of stream copy
-	for (i=0; i<nb_output_files; i++) {
-		dump_format(output_files[i], i, output_files[i]->filename, 1);
-	}
-
-	/* dump the stream mapping */
-	if (verbose >= 0) {
-		wxLogMessage(wxT("Stream mapping:"));
-		for (i=0; i<nb_ostreams; i++) {
-			ost = ost_table[i];
-			wxString syncStr;
-			if (ost->sync_ist != ist_table[ost->source_index])
-				syncStr = wxString::Format(wxT(" [sync #%d.%d]"),
-				ost->sync_ist->file_index,
-				ost->sync_ist->index);
-			wxLogMessage(wxT("  Stream #%d.%d -> #%d.%d%s"),
-			ist_table[ost->source_index]->file_index,
-			ist_table[ost->source_index]->index,
-			ost->file_index, ost->index, syncStr.c_str());
-		}
 	}
 
 	/* open each encoder */
@@ -1185,14 +1142,16 @@ bool wxFfmpegMediaTranscoder::Run(bool& canceled) {
 			if (!codec)
 				codec = avcodec_find_encoder(ost->st->codec->codec_id);
 			if (!codec) {
-				wxLogError(wxT("Unsupported codec for output stream #%d.%d"),
+				error = wxString::Format(wxT("Unsupported codec for output stream #%d.%d"),
 						ost->file_index, ost->index);
-				return false;
+				ret = AVERROR(EINVAL);
+				goto dump_format;
 			}
 			if (avcodec_open(ost->st->codec, codec) < 0) {
-				wxLogError(wxT("Error while opening codec for output stream #%d.%d - maybe incorrect parameters such as bit_rate, rate, width or height"),
+				error = wxString::Format(wxT("Error while opening codec for output stream #%d.%d - maybe incorrect parameters such as bit_rate, rate, width or height"),
 						ost->file_index, ost->index);
-				return false;
+				ret = AVERROR(EINVAL);
+				goto dump_format;
 			}
 			extra_size += ost->st->codec->extradata_size;
 		}
@@ -1229,35 +1188,32 @@ bool wxFfmpegMediaTranscoder::Run(bool& canceled) {
 
 	/* set meta data information from input file if required */
 	for (i=0; i<nb_meta_data_maps; i++) {
+		AVMetadataTag *mtag;
 		AVFormatContext *out_file;
 		AVFormatContext *in_file;
 
 		int out_file_index = meta_data_maps[i].out_file;
 		int in_file_index = meta_data_maps[i].in_file;
 		if (out_file_index < 0 || out_file_index >= nb_output_files) {
-			wxLogError(wxT("Invalid output file index %d map_meta_data(%d,%d)"),
-			out_file_index, out_file_index, in_file_index);
+			error = wxString::Format(wxT("Invalid output file index %d map_meta_data(%d,%d)"),
+					out_file_index, out_file_index, in_file_index);
 			ret = AVERROR(EINVAL);
-			goto fail;
+			goto dump_format;
 		}
 		if (in_file_index < 0 || in_file_index >= nb_input_files) {
-			wxLogError(wxT("Invalid input file index %d map_meta_data(%d,%d)"),
-			in_file_index, out_file_index, in_file_index);
+			error = wxString::Format(wxT("Invalid input file index %d map_meta_data(%d,%d)"),
+					in_file_index, out_file_index, in_file_index);
 			ret = AVERROR(EINVAL);
-			goto fail;
+			goto dump_format;
 		}
 
 		out_file = output_files[out_file_index];
 		in_file = input_files[in_file_index];
-
-		strcpy(out_file->title, in_file->title);
-		strcpy(out_file->author, in_file->author);
-		strcpy(out_file->copyright, in_file->copyright);
-		strcpy(out_file->comment, in_file->comment);
-		strcpy(out_file->album, in_file->album);
-		out_file->year = in_file->year;
-		out_file->track = in_file->track;
-		strcpy(out_file->genre, in_file->genre);
+		
+		mtag=NULL;
+		while((mtag=av_metadata_get(in_file->metadata, "", mtag, AV_METADATA_IGNORE_SUFFIX)))
+			av_metadata_set(&out_file->metadata, mtag->key, mtag->value);
+		av_metadata_conv(out_file, out_file->oformat->metadata_conv, in_file->iformat->metadata_conv);
 	}
 
 	/* open files and write file headers */
@@ -1266,12 +1222,38 @@ bool wxFfmpegMediaTranscoder::Run(bool& canceled) {
 		if (av_write_header(os) < 0) {
 			wxLogError(wxT("Could not write header for output file #%d (incorrect codec parameters ?)"), i);
 			ret = AVERROR(EINVAL);
-			goto fail;
+			goto dump_format;
 		}
 		if (strcmp(output_files[i]->oformat->name, "rtp")) {
 			want_sdp = 0;
 		}
 	}
+	
+dump_format:
+	/* dump the file output parameters - cannot be done before in case
+	   of stream copy */
+	for(i=0;i<nb_output_files;i++) {
+	    dump_format(output_files[i], i, output_files[i]->filename, 1);
+	}
+
+	/* dump the stream mapping */
+	if (verbose >= 0) {
+	    wxLogMessage(wxT("Stream mapping:"));
+	    for(i=0;i<nb_ostreams;i++) {
+	        ost = ost_table[i];
+	        wxString msg = wxString::Format(wxT("  Stream #%d.%d -> #%d.%d"),
+	                ist_table[ost->source_index]->file_index, ist_table[ost->source_index]->index,
+	                ost->file_index, ost->index);
+	        if (ost->sync_ist != ist_table[ost->source_index])
+	        	msg += wxString::Format(wxT(" [sync #%d.%d]"), ost->sync_ist->file_index, ost->sync_ist->index);
+	        wxLogError(msg);
+	    }
+	}
+	if (ret) {
+		wxLogError(error);
+	    goto fail;
+	}
+	
 //    if (want_sdp) {
 //        print_sdp(output_files, nb_output_files);
 //    }
@@ -1300,6 +1282,8 @@ bool wxFfmpegMediaTranscoder::Run(bool& canceled) {
 			ost = ost_table[i];
 			os = output_files[ost->file_index];
 			ist = ist_table[ost->source_index];
+			if(no_packet[ist->file_index])
+				continue;
 			if (ost->st->codec->codec_type == CODEC_TYPE_VIDEO)
 				opts = ost->sync_opts * av_q2d(ost->st->codec->time_base);
 			else
@@ -1324,6 +1308,12 @@ bool wxFfmpegMediaTranscoder::Run(bool& canceled) {
 		}
 		/* if none, if is finished */
 		if (file_index < 0) {
+			if(no_packet_count){
+				no_packet_count=0;
+				memset(no_packet, 0, sizeof(no_packet));
+				wxMicroSleep(10000);
+				continue;
+			}
 			break;
 		}
 
@@ -1337,7 +1327,13 @@ bool wxFfmpegMediaTranscoder::Run(bool& canceled) {
 
 		/* read a frame from it and output it in the fifo */
 		is = input_files[file_index];
-		if (av_read_frame(is, &pkt) < 0) {
+		ret= av_read_frame(is, &pkt);
+		if(ret == AVERROR(EAGAIN)){
+			no_packet[file_index]=1;
+			no_packet_count++;
+			continue;
+		}
+		if (ret < 0) {
 			file_table[file_index].eof_reached = 1;
 			if (opt_shortest)
 				break;
@@ -1345,6 +1341,13 @@ bool wxFfmpegMediaTranscoder::Run(bool& canceled) {
 				continue;
 		}
 		
+		no_packet_count=0;
+		memset(no_packet, 0, sizeof(no_packet));
+		
+//		if (do_pkt_dump) {
+//			av_pkt_dump_log(NULL, AV_LOG_DEBUG, &pkt, do_hex_dump);
+//		}
+		
 		/* the following test is needed in case new streams appear
 		 dynamically in stream : we ignore them */
 		if (pkt.stream_index >= file_table[file_index].nb_streams)
@@ -1433,7 +1436,7 @@ bool wxFfmpegMediaTranscoder::Run(bool& canceled) {
 	
 	/* finished ! */
 	ret = 0;
-fail1:
+fail:
 	av_freep(&bit_buffer);
 	av_free(file_table);
 	
@@ -1452,8 +1455,12 @@ fail1:
 					fclose(ost->logfile);
 					ost->logfile = NULL;
 				}
-				av_fifo_free(&ost->fifo); /* works even if fifo is not
-				 initialized but set to zero */
+#if LIBAVUTIL_VERSION_INT > AV_VERSION_INT(49, 15, 0)
+				av_fifo_free(ost->fifo); /* works even if fifo is not initialized but set to zero */
+#else
+				if (ost->fifo)
+					av_free(ost->fifo);
+#endif
 				av_free(ost->pict_tmp.data[0]);
 				if (ost->video_resample)
 					sws_freeContext(ost->img_resample_ctx);
@@ -1469,9 +1476,6 @@ fail1:
 	if (ret == 0)
 		End();
 	return ret == 0;
-fail:
-	ret = AVERROR(ENOMEM);
-	goto fail1;
 }
 
 /* pkt = NULL means EOF (needed to flush decoder buffers) */
@@ -1564,8 +1568,9 @@ bool wxFfmpegMediaTranscoder::OutputPacket(AVInputStream *ist,
 					goto discard_packet;
 				}
 				if (ist->st->codec->time_base.num != 0) {
+					int ticks= ist->st->parser ? ist->st->parser->repeat_pict+1 : ist->st->codec->ticks_per_frame;
 					ist->next_pts += ((int64_t)AV_TIME_BASE
-							* ist->st->codec->time_base.num)
+							* ist->st->codec->time_base.num * ticks)
 							/ ist->st->codec->time_base.den;
 				}
 				len = 0;
@@ -1593,8 +1598,9 @@ bool wxFfmpegMediaTranscoder::OutputPacket(AVInputStream *ist,
 				break;
 			case CODEC_TYPE_VIDEO:
 				if (ist->st->codec->time_base.num != 0) {
+					int ticks= ist->st->parser ? ist->st->parser->repeat_pict+1 : ist->st->codec->ticks_per_frame;
 					ist->next_pts += ((int64_t)AV_TIME_BASE
-							* ist->st->codec->time_base.num)
+							* ist->st->codec->time_base.num * ticks)
 							/ ist->st->codec->time_base.den;
 				}
 				break;
@@ -1774,13 +1780,17 @@ discard_packet:
 
 						switch (ost->st->codec->codec_type) {
 						case CODEC_TYPE_AUDIO:
-							fifo_bytes = av_fifo_size(&ost->fifo);
+							fifo_bytes = av_fifo_size(ost->fifo);
 							ret = 0;
 							/* encode any samples remaining in fifo */
 							if (fifo_bytes > 0 && enc->codec->capabilities & CODEC_CAP_SMALL_LAST_FRAME) {
 								int fs_tmp = enc->frame_size;
 								enc->frame_size = fifo_bytes / (2 * enc->channels);
-								av_fifo_read(&ost->fifo, (uint8_t *)samples, fifo_bytes);
+#if LIBAVUTIL_VERSION_INT > AV_VERSION_INT(49, 15, 0)
+								av_fifo_generic_read(ost->fifo, samples, fifo_bytes, NULL);
+#else
+								av_fifo_generic_read(ost->fifo, fifo_bytes, NULL, samples);
+#endif
 								ret	= avcodec_encode_audio(enc, bit_buffer, bit_buffer_size, samples);
 								enc->frame_size = fs_tmp;
 							}
@@ -1893,13 +1903,12 @@ bool wxFfmpegMediaTranscoder::DoAudioOut(AVFormatContext *s, AVOutputStream *ost
 		ost->audio_resample = 1;
 	
 	if (ost->audio_resample && !ost->resample) {
-		 if (dec->sample_fmt != SAMPLE_FMT_S16) {
-			wxLogError(wxT("Audio resampler only works with 16 bits per sample, patch welcome."));
-			return false;
-        }
-		ost->resample = audio_resample_init(
-				enc->channels, dec->channels,
-				enc->sample_rate, dec->sample_rate);
+		if (dec->sample_fmt != SAMPLE_FMT_S16)
+			wxLogWarning(wxT("Warning, using s16 intermediate sample format for resampling"));
+		ost->resample = av_audio_resample_init(enc->channels, dec->channels,
+				enc->sample_rate, dec->sample_rate,
+				enc->sample_fmt, dec->sample_fmt,
+				16, 10, 0, 0.8);
 		if (!ost->resample) {
 			wxLogError(wxT("Can not resample %d channels @ %d Hz to %d channels @ %d Hz"),
 					dec->channels, dec->sample_rate,
@@ -1908,7 +1917,7 @@ bool wxFfmpegMediaTranscoder::DoAudioOut(AVFormatContext *s, AVOutputStream *ost
 		}
 	}
 	
-//	if (dec->sample_fmt!=enc->sample_fmt &&
+//	if (!ost->audio_resample && dec->sample_fmt!=enc->sample_fmt &&
 //		MAKE_SFMT_PAIR(enc->sample_fmt,dec->sample_fmt)!=ost->reformat_pair) {
 //		if (!audio_out2)
 //			audio_out2 = (uint8_t*) av_malloc(audio_out_size);
@@ -1929,7 +1938,7 @@ bool wxFfmpegMediaTranscoder::DoAudioOut(AVFormatContext *s, AVOutputStream *ost
 	
 	if (audio_sync_method) {
 		double delta = get_sync_ipts(ost, start_time) * enc->sample_rate - ost->sync_opts
-				- av_fifo_size(&ost->fifo) / (ost->st->codec->channels * 2);
+				- av_fifo_size(ost->fifo) / (ost->st->codec->channels * 2);
 		double idelta= delta * ist->st->codec->sample_rate / enc->sample_rate;
 		int byte_delta= ((int)idelta)*2*ist->st->codec->channels;
 		
@@ -1976,7 +1985,7 @@ bool wxFfmpegMediaTranscoder::DoAudioOut(AVFormatContext *s, AVOutputStream *ost
 		}
 	} else
 		ost->sync_opts= (int64_t) rint(get_sync_ipts(ost, start_time) * enc->sample_rate)
-				- av_fifo_size(&ost->fifo)/(ost->st->codec->channels * 2); //FIXME wrong
+				- av_fifo_size(ost->fifo)/(ost->st->codec->channels * 2); //FIXME wrong
 	
 	if (ost->audio_resample) {
 		buftmp = audio_buf;
@@ -1989,7 +1998,7 @@ bool wxFfmpegMediaTranscoder::DoAudioOut(AVFormatContext *s, AVOutputStream *ost
 		size_out = size;
 	}
 	
-//	if (dec->sample_fmt!=enc->sample_fmt) {
+//	if (!ost->audio_resample && dec->sample_fmt!=enc->sample_fmt) {
 //		const void *ibuf[6]= {buftmp};
 //		void *obuf[6]= {audio_out2};
 //		int istride[6]= {isize};
@@ -2008,19 +2017,23 @@ bool wxFfmpegMediaTranscoder::DoAudioOut(AVFormatContext *s, AVOutputStream *ost
 	/* now encode as many frames as possible */
 	if (enc->frame_size > 1) {
 		/* output resampled raw samples */
-		if (av_fifo_realloc2(&ost->fifo, av_fifo_size(&ost->fifo) + size_out) < 0) {
+		if (av_fifo_realloc2(ost->fifo, av_fifo_size(ost->fifo) + size_out) < 0) {
             wxLogError(wxT("av_fifo_realloc2() failed"));
             return false;
         }
-        av_fifo_generic_write(&ost->fifo, buftmp, size_out, NULL);
+        av_fifo_generic_write(ost->fifo, buftmp, size_out, NULL);
 
         frame_bytes = enc->frame_size * osize * enc->channels;
 
-        while (av_fifo_size(&ost->fifo) >= frame_bytes) {
+        while (av_fifo_size(ost->fifo) >= frame_bytes) {
 			AVPacket pkt;
 			av_init_packet(&pkt);
 			
-			av_fifo_read(&ost->fifo, audio_buf, frame_bytes);
+#if LIBAVUTIL_VERSION_INT > AV_VERSION_INT(49, 15, 0)
+			av_fifo_generic_read(ost->fifo, audio_buf, frame_bytes, NULL);
+#else
+			av_fifo_generic_read(ost->fifo, frame_bytes, NULL, audio_buf);
+#endif
 
             //FIXME pass ost->sync_opts as AVFrame.pts in avcodec_encode_audio()
 			
@@ -2117,11 +2130,14 @@ bool wxFfmpegMediaTranscoder::DoVideoOut(AVFormatContext *s, AVOutputStream *ost
 		//FIXME set to 0.5 after we fix some dts/pts bugs like in avidec.c
 		if (vdelta < -1.1)
 			nb_frames = 0;
-		else if (video_sync_method == 2)
+		 else if (video_sync_method == 2 || (video_sync_method < 0 && (s->oformat->flags & AVFMT_VARIABLE_FPS))) {
+			if (vdelta <= -0.6) {
+				nb_frames = 0;
+			} else if (vdelta > 0.6)
 			ost->sync_opts= lrintf(get_sync_ipts(ost, start_time) / av_q2d(enc->time_base));
-		else if (vdelta > 1.1)
+		 } else if (vdelta > 1.1)
 			nb_frames = (int) rint(vdelta);
-		//wxLogMessage(wxT("vdelta:%f, ost->sync_opts:%"PRId64"), ost->sync_ipts:%f nb_frames:%d\n", vdelta, ost->sync_opts, ost->sync_ipts, nb_frames);
+		//wxLogMessage(wxT("vdelta:%f, ost->sync_opts:%"PRId64"), ost->sync_ipts:%f nb_frames:%d\n", vdelta, ost->sync_opts, get_sync_ipts(ost), nb_frames);
 		if (nb_frames == 0) {
 			++nb_frames_drop;
 			if (verbose>2)
@@ -2144,7 +2160,7 @@ bool wxFfmpegMediaTranscoder::DoVideoOut(AVFormatContext *s, AVOutputStream *ost
 				(AVPicture *)&picture_crop_temp,
 				(AVPicture *)in_picture, dec->pix_fmt,
 				ost->topBand, ost->leftBand) < 0) {
-			av_log(NULL, AV_LOG_ERROR, "error cropping picture\n");
+			wxLogError(wxT("error cropping picture"));
 			if (exit_on_error)
 				return false;
 			return true;
@@ -2165,7 +2181,7 @@ bool wxFfmpegMediaTranscoder::DoVideoOut(AVFormatContext *s, AVOutputStream *ost
 					(AVPicture *)final_picture,
 					enc->pix_fmt, ost->padtop,
 					ost->padleft) < 0) {
-				av_log(NULL, AV_LOG_ERROR, "error padding picture\n");
+				wxLogError(wxT("error padding picture"));
 				if (exit_on_error)
 					return false;
 				return true;
@@ -2449,10 +2465,10 @@ void wxFfmpegMediaTranscoder::End() {
 			av_free(s->streams[j]);
 		}
 #if LIBAVFORMAT_VERSION_INT >= AV_VERSION_INT(52, 24, 1)
-		for(int j=0;j<s->nb_programs;j++) {
+		for(int j=0;j<(int)s->nb_programs;j++) {
 			av_metadata_free(&s->programs[j]->metadata);
 		}
-		for(int j=0;j<s->nb_chapters;j++) {
+		for(int j=0;j<(int)s->nb_chapters;j++) {
 			av_metadata_free(&s->chapters[j]->metadata);
 		}
 		av_metadata_free(&s->metadata);
